package controll;

import gui.Dialog;
import gui.OpenDialog;
import gui.report.CaseEditor;
import gui.report.CaseEditorMainPanel;
import gui.report.DescriptionPanel;
import gui.report.ReportTab;
import io.SQLDBAccess;
import io.XMLAccess;
import io.XMLDBAccess;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.coordination.ServiceProvider;
import services.suchen.FortschrittInfo;
import xml.Case;
import xml.Constants;

public class CaseEditorControll implements ServiceProvider {

	private static final Map<CaseEditor, Case> openendReports = new java.util.Hashtable<CaseEditor, Case>();

	private static CaseEditorMainPanel mainp;

	public static void init(CaseEditorMainPanel main) {
		mainp = main;
	}

	public static Case getOpenedReport(CaseEditor edit) {
		return openendReports.get(edit);
	}

	public static void addNewReport(CaseEditor editor) {
		String caption = (editor.getReports().size() == 0) ? "Befund" : (editor
				.getReports().size() + 1)
				+ "._Bericht";
		editor.addReport(new ReportTab(editor), caption);
	}

	public static void createSchnellSchnittReport(CaseEditor editor) {
		if (!openendReports.get(editor).isSchnellSchnitt()
				&& editor.getReports().size() == 0)
			editor.addReport(new ReportTab(editor), "Schnellschnitt");
	}

	public static void addImage(CaseEditor editor) {
		JFileChooser fc = Dialog.generateImageChooser();
		if (fc.showOpenDialog(fc) == JFileChooser.APPROVE_OPTION) {
	//		ImageViewer view = editor.getImageviewer();
			// view.
		}
	}

	public static void openRemoteCase() {

		String[] initList = XMLDBAccess.listStoredFiles();
		if (initList.length != 0) {

			OpenDialog dia = Dialog.generateDBFileChooser(initList);

			dia.setVisible(true);

			if (dia.okButtonPresses()) {
				int index = dia.getSelectedIndex();
				if (index != -1) {
					MainControll.SERVICECOORDINATOR.requestService(
							ServiceID.SHOW_PROGRESS,
							new Parameter(null,
							
							new FortschrittInfo(true,
									"Datei oeffnen")));
					String id = dia.getElementAt(index);
					Document d = XMLDBAccess.loadCaseFromDB(id);
					if(d == null)
						return;
					Case current = new Case(d);
					CaseEditor temp;
					openendReports.put(temp = new CaseEditor(), current);
					temp.init();
					updateView(temp);
					mainp.openNewReportEditor(temp, id);
					mainp.getTabbedPane().setSelectedIndex(
							mainp.getTabbedPane().getComponentCount() - 1);

					MainControll.SERVICECOORDINATOR.requestService(
							ServiceID.SHOW_PROGRESS,
							new Parameter(null,
							
							new FortschrittInfo(false)));
				
				}
			}
		}
	}

	public static void saveRemoteCase(CaseEditor edit, String name) {
		updateModel(edit);
		edit.getTitle();
		MainControll.SERVICECOORDINATOR.requestService(
				ServiceID.SHOW_PROGRESS,
				new Parameter(null,
				
				new FortschrittInfo(true,
						"Datei speichern")));
		
		XMLDBAccess.saveCaseToDB(name, getOpenedReport(edit).getDocument());
		
		MainControll.SERVICECOORDINATOR.requestService(
				ServiceID.SHOW_PROGRESS,
				new Parameter(null,
				
				new FortschrittInfo(false)));
		
	}

	public static void createNewCase() {
		Case current = new Case(null);
		CaseEditor temp;
		openendReports.put(temp = new CaseEditor(), current);
		temp.init();
		mainp.openNewReportEditor(temp, "Neuer Bericht");
	}

	public static void saveLocalCase(CaseEditor editor) {
		JFileChooser fc = Dialog.generateFileChooser();
		if (fc.showSaveDialog(fc) == JFileChooser.APPROVE_OPTION) {
			File f = new File(fc.getSelectedFile().getAbsolutePath().endsWith(
					".xml") ? fc.getSelectedFile().getAbsolutePath() : fc
					.getSelectedFile().getAbsolutePath()
					+ ".xml");
			if (f.exists()) {
				int value = JOptionPane.showConfirmDialog(null,
						"Die Datei wirklich überschreiben?",
						"Datei überschreiben", JOptionPane.YES_NO_OPTION);
				switch (value) {
				case JOptionPane.OK_OPTION:
					break;
				case JOptionPane.CLOSED_OPTION:
				case JOptionPane.NO_OPTION:
					return;
				}
			}

			MainControll.SERVICECOORDINATOR.requestService(
					ServiceID.SHOW_PROGRESS,
					new Parameter(null,
					
					new FortschrittInfo(true,
							"Datei speichern")));
			try {
				updateModel(editor);
				XMLAccess.exportCase(f, openendReports.get(editor)
						.getDocument());
				editor.setModifiedState(false);
			} catch (IOException e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"could not save the XML Schema", "Error",
						JOptionPane.ERROR_MESSAGE);
			}
			
			

			MainControll.SERVICECOORDINATOR.requestService(
					ServiceID.SHOW_PROGRESS,
					new Parameter(null,
					
					new FortschrittInfo(false)));
			
		}
	}

	public static void openLocalCase() {
		JFileChooser fc = Dialog.generateFileChooser();
		if (fc.showOpenDialog(fc) == JFileChooser.APPROVE_OPTION) {
			MainControll.SERVICECOORDINATOR.requestService(
					ServiceID.SHOW_PROGRESS,
					new Parameter(null,
					
					new FortschrittInfo(true,
							"Detei oeffnen")));
			
			try {
				Case current = new Case(XMLAccess.importCase(fc
						.getSelectedFile()));
				CaseEditor temp;
				openendReports.put(temp = new CaseEditor(), current);
				temp.init();
				updateView(temp);
				mainp.openNewReportEditor(temp, fc.getSelectedFile().getName());
				mainp.getTabbedPane().setSelectedIndex(
						mainp.getTabbedPane().getComponentCount() - 1);
			} catch (ParserConfigurationException e) {
				e.printStackTrace();
			} catch (SAXException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			MainControll.SERVICECOORDINATOR.requestService(
					ServiceID.SHOW_PROGRESS,
					new Parameter(null,
					
					new FortschrittInfo(false)));
			
		}
	}

	public static void closeEditor(CaseEditor editor) {
		openendReports.remove(editor);
		editor.dispose();
	}

	private static void updateView(CaseEditor ce) {
		Case c = openendReports.get(ce);
		Vector<ReportTab> reports = ce.getReports();
		DescriptionPanel description = ce.getDescriptionPanel();
		for (int i = 0; i < reports.size(); i++) {
			for (int j = 0; j < Constants.REPORT_DATA.length; j++)
				reports.elementAt(i).setValue(Constants.REPORT_DATA[j],
						c.getContent(Constants.REPORT_DATA[j], i));
		}

		for (int i = 0; i < Constants.CASE_DATA.length; i++) {
			description.setValue(Constants.CASE_DATA[i], c
					.getNodeValue(Constants.CASE_DATA[i]));
		}

		for (int i = 0; i < Constants.PATIENT_DATA.length; i++) {
			if (Constants.PATIENT_DATA[i]
					.equals(Constants.PATIENT_KOSTENTRAEGER))
				description.setValue(Constants.PATIENT_KOSTENTRAEGER, c
						.getKostentraeger());
			else if (Constants.PATIENT_DATA[i]
					.equals(Constants.PATIENT_ENTRY_NUMBER))
				description.setValue(Constants.PATIENT_ENTRY_NUMBER, c
						.getEnumber());
			else
				description.setValue(Constants.PATIENT_DATA[i], c
						.getNodeValue(Constants.PATIENT_DATA[i]));
		}
		updateStatistic(c, description);
		updateQuality(c, description, reports.size());
	}

	private static void updateQuality(Case c, DescriptionPanel description,
			int reportcount) {
		if (c.getNodeValue(Constants.PATIENT_PATIENT_ID) != null)
			description.setValue(Constants.QUALITY_PATIENT_ID, "Ja");
		if (c.getNodeValue(Constants.PATIENT_PATIENT_NAME) != null
				&& c.getNodeValue(Constants.PATIENT_PATIENT_BIRTH) != null
				&& c.getNodeValue(Constants.PATIENT_PATIENT_GENDER) != null)
			description.setValue(Constants.QUALITY_DEMOGRAPHIC_VALUES, "Ja");
		else if (c.getNodeValue(Constants.PATIENT_PATIENT_NAME) == null
				&& c.getNodeValue(Constants.PATIENT_PATIENT_BIRTH) == null
				&& c.getNodeValue(Constants.PATIENT_PATIENT_GENDER) == null)
			description.setValue(Constants.QUALITY_DEMOGRAPHIC_VALUES, "Nein");
		else
			description.setValue(Constants.QUALITY_DEMOGRAPHIC_VALUES,
					"Unvollstaendig");
		if (c.getNodeValue(Constants.CASE_CASE_ID) != null)
			description.setValue(Constants.QUALITY_CASE_ID, "Ja");
		if (c.getEnumber() != null)
			description.setValue(Constants.QUALITY_ENTRY_NUMBER, "Ja");
		if (c.getNodeValue(Constants.CASE_CASE_DATE) != null)
			description.setValue(Constants.QUALITY_CASE_DATE, "Ja");
		if (c.getNodeValue(Constants.CASE_SENDER) != null)
			description.setValue(Constants.QUALITY_SENDER, "Ja");
		if (c.getKostentraeger() != null)
			description.setValue(Constants.QUALITY_KOSTENTRAEGER, "Ja");
		if (c.getNodeValue(Constants.CASE_REPORT_ID) != null)
			description.setValue(Constants.QUALITY_REPORT_ID, "Ja");
		if (c.getNodeValue(Constants.CASE_REPORT_DATE) != null)
			description.setValue(Constants.QUALITY_REPORT_DATE, "Ja");
		if (c.getNodeValue(Constants.CASE_FIRST_DOCTOR) != null
				&& c.getNodeValue(Constants.CASE_SECOND_DOCTOR) != null
				&& c.getNodeValue(Constants.CASE_THIRD_DOCTOR) != null)
			description.setValue(Constants.QUALITY_DOCTORS, "Ja");

		for (int i = 0; i < reportcount; i++) {
			if (c.getContent("Mikroskopie", i) != null)
				description.setValue(Constants.QUALITY_MIKRO, "Ja");
			if (c.getContent("Makroskopie", i) != null)
				description.setValue(Constants.QUALITY_MAKRO, "Ja");
			if (c.getContent("Kommentar", i) != null)
				description.setValue(Constants.QUALITY_COMMENT, "Ja");
			if (c.getContent("Kritischer_Bericht", i) != null)
				description.setValue(Constants.QUALITY_CRITICAL, "Ja");
		}
		if (c.isSchnellSchnitt())
			description.setValue(Constants.QUALITY_SCNELLSCHNITT, "Ja");
	}

	private static void updateStatistic(Case c, DescriptionPanel description) {

		description.setValue(Constants.STATISTIC_NUMBEROFREPORTS, SQLDBAccess
				.getNumberofReports(c
						.getNodeValue(Constants.PATIENT_PATIENT_ID)));
		description.setValue(Constants.PATIENT_NUMBEROFDIAGNOSIS, SQLDBAccess
				.getDiagnosis(c.getNodeValue(Constants.CASE_CASE_ID)));
	}

	private static void updateModel(CaseEditor ce) {
		Case c = openendReports.get(ce);
		Vector<ReportTab> reports = ce.getReports();
		DescriptionPanel description = ce.getDescriptionPanel();

		for (int i = 0; i < reports.size(); i++) {
			c.setSection(ce.getCaption(i), i);

			for (int j = 0; j < Constants.REPORT_DATA.length; j++)
				c.setContent(Constants.REPORT_DATA[j], i, reports.elementAt(i)
						.getValue(Constants.REPORT_DATA[j]));

		}

		for (int i = 0; i < Constants.CASE_DATA.length; i++) {
			c.setNodeValue(Constants.CASE_DATA[i], description
					.getValue(Constants.CASE_DATA[i]));
		}

		for (int i = 0; i < Constants.PATIENT_DATA.length; i++) {
			if (Constants.PATIENT_DATA[i]
					.equals(Constants.PATIENT_KOSTENTRAEGER))
				description.setValue(Constants.PATIENT_KOSTENTRAEGER, c
						.getKostentraeger());
			else if (Constants.PATIENT_DATA[i]
					.equals(Constants.PATIENT_ENTRY_NUMBER))
				description.setValue(Constants.PATIENT_ENTRY_NUMBER, c
						.getEnumber());
			else
				description.setValue(Constants.PATIENT_DATA[i], c
						.getNodeValue(Constants.PATIENT_DATA[i]));
		}

	}

	public void performService(ServiceID sid, Parameter param) {
		if (sid.equals(ServiceID.SHOW_UI_REPORT)) {
			String id = (String)(param.value);
			Document d = XMLDBAccess.loadCaseFromDB(id +".xml");
			if(d == null)
				return;
			Case current = new Case(d);
			CaseEditor temp;
			openendReports.put(temp = new CaseEditor(), current);
			temp.init();
			updateView(temp);
			mainp.openNewReportEditor(temp, id+".xml");
			mainp.getTabbedPane().setSelectedIndex(
					mainp.getTabbedPane().getComponentCount() - 1);
			
			MainControll.showCaseEditorPanel();
			
		}

	}

}
