package gui.report;

import gui.report.image.ImageViewer;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import xml.Case;
import controll.CaseEditorControll;

public class CaseEditor extends JInternalFrame {

	private boolean modified = false;

	private JTabbedPane data;

	// dirty workaround for the listeners
	private CaseEditor t = this;

	private DescriptionPanel description;

	private ImageViewer images;
	
	private Vector<ReportTab> reports;

	public CaseEditor() {
		this.reports = new Vector<ReportTab>();
		this.setContentPane(new JPanel(new BorderLayout()));
		this.getContentPane().add(data = new JTabbedPane(), "Center");
		this.getContentPane().add(createEditButtons(), "North");
		this.setBorder(BorderFactory.createEtchedBorder());
		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent arg0) {
				if (modified) {
					int value = JOptionPane.showConfirmDialog(null,
							"Wirklich schliessen ohne zu speichern?",
							"ReportEditor", JOptionPane.YES_NO_OPTION);
					switch (value) {
					case JOptionPane.CLOSED_OPTION:
					case JOptionPane.NO_OPTION:
						break;
					case JOptionPane.OK_OPTION:
						close();
						break;
					}
				} else
					close();
			}
		});
		this.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		this.setClosable(true);
	}

	private void close() {
		CaseEditorControll.closeEditor(this);
	}

	private JPanel createEditButtons() {
		JPanel temp = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JButton createSchnellSchnitt, createReport, addImage;
		temp.add(createSchnellSchnitt = new JButton("Schnellschnitt erzeugen"));
		temp.add(createReport = new JButton("Bericht erzeugen"));
		temp.add(addImage = new JButton("Bild hinzufuegen"));
		createSchnellSchnitt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				CaseEditorControll.createSchnellSchnittReport(t);
			}
		});
		createReport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				CaseEditorControll.addNewReport(t);

			};
		});
		addImage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				CaseEditorControll.addImage(t);
			}
		});
		return temp;
	}

	public DescriptionPanel getDescriptionPanel() {
		return description;
	}

	public Vector<ReportTab> getReports() {
		return reports;
	}

	public void setModifiedState(boolean state) {
		modified = state;
	}
	
	public ImageViewer getImageviewer(){
		return images;
	}
	

	public void init() {
		Case c = CaseEditorControll.getOpenedReport(this);
		List<String> captions = c.getCaptions();
		data.add(description = new DescriptionPanel(), "Befunddaten");
		for(Iterator<String> i = captions.iterator(); i.hasNext();){
			ReportTab temp = new ReportTab(this);
			reports.add(temp);
			data.add(temp, i.next());
		}
		
		String path = 
			//"file:///Z:/eclipseworkspace/swpathoold/classes/images"
			"http://swpatho.ag-nbi.de/swpatho/img/"
			
			;
		
		
		Vector<String> files = new Vector<String>();
		files.add(path +"7FF123_Adenokarzinom.jpg");
		files.add(path + "7FF123_Adenokarzinom_mikro.jpg");
		files.add(path + "7FF103_interstitielle_Pneumonie.jpg");
		
	//	files.add("http://www.inf.fu-berlin.de/styles/inst-title-600x400.jpg");
		
		
		Vector<String> thumbs = new Vector<String>();
		thumbs.add(path + "7FF123_Adenokarzinom_th.jpg");
		thumbs.add(path + "7FF123_Adenokarzinom_mikro_th.jpg");
		thumbs.add(path+"7FF123_interstitielle_Pneumonie_th.jpg");
		//thumbs.add("http://www.inf.fu-berlin.de/styles/inst-title-600x400.jpg");
		
		data.add(images = new ImageViewer(files,thumbs), "Bilder");
	}

	public String getCaption(int position) {
		if (position >= 0 && position < reports.size())
			return data.getTitleAt(position + 1);
		else
			return null;
	}

	public void addReport(ReportTab tab, String caption) {
		reports.add(tab);
		data.add(tab, caption);
	}
}
