package gui.report;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;

import xml.Constants;

/**
 * <p>
 * Title: ReportTab
 * </p>
 * <p>
 * Description: Editor f�r Befundberichte - FU Berlin 2006
 * </p>
 * <p>
 * Class Description: Paints a tab to represent a befund
 * </p>
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author harasic
 * @version 1.0
 */

public class ReportTab extends JPanel {

	private static final long serialVersionUID = 1025491259039135894L;

	private JLabel Zeit = new JLabel(), BildIDLabel = new JLabel(),
			startzeit = new JLabel(), stopzeit = new JLabel(),
			bildId = new JLabel();

	protected JButton startButton = new JButton(), stopButton = new JButton();

	private CaseEditor parent;

	private Hashtable<String, JTextArea> values;

	public ReportTab(CaseEditor parent) {
		this.parent = parent;
		values = new Hashtable<String, JTextArea>();
		this.setLayout(new GridBagLayout());
		this.setAlignmentY(0.5f);
		this.setBorder(BorderFactory.createEtchedBorder());
		// Case c = Controll.getOpenedReport(parent);
		// List<String> sections = c.getSections(report);

		for (int i = 0; i < Constants.REPORT_DATA.length; i++) {
			JLabel templabel = new JLabel(Constants.REPORT_DATA[i]);
			templabel.setFont(new java.awt.Font("Dialog", 1, 14));
			templabel.setForeground(Color.BLUE);
			//templabel.setPreferredSize(new Dimension(60, 15));

			JTextArea temparea = new JTextArea();
			values.put(Constants.REPORT_DATA[i], temparea);
			temparea.setEditable(true);
			temparea.setMargin(new Insets(0, 0, 0, 0));
			temparea.setLineWrap(true);
			temparea.setTabSize(8);
			temparea.setWrapStyleWord(true);
			temparea.addKeyListener(new KeyAdapter() {
				public void keyTyped(KeyEvent arg0) {
					setModified();
				};
			});

			JScrollPane tempscrollpane = new JScrollPane();
			tempscrollpane
					.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
			tempscrollpane.setAutoscrolls(true);
			// scrollpanes[i].setBorder(BorderFactory.createEtchedBorder());
			tempscrollpane.setMinimumSize(new Dimension(24, 24));
			tempscrollpane.setPreferredSize(new Dimension(8, 25));
			tempscrollpane.setVerifyInputWhenFocusTarget(true);
			
			this.add(templabel, new GridBagConstraints(0, i, 1, 1, 0.0, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.BOTH,
					new Insets(0, 10, 0, 0), 0, 0));
			
			this.add(tempscrollpane, new GridBagConstraints(1, i, 5, 1, 1.0,
					1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
					new Insets(5, 5, 5, 20), 0, 0));
			tempscrollpane.getViewport().add(temparea, null);

			
		}

	/*	Zeit.setFont(new java.awt.Font("Dialog", 1, 14));
		Zeit.setText("Zeit");

		BildIDLabel.setRequestFocusEnabled(true);
		BildIDLabel.setToolTipText("Bild-ID");
		BildIDLabel.setHorizontalAlignment(SwingConstants.LEFT);
		BildIDLabel.setText("Bild-ID");
		stopzeit.setHorizontalAlignment(SwingConstants.LEFT);
		startButton.setText("Start");
		stopButton.setText("Stop");
		startzeit.setText("");
		bildId.setText("");

		this.add(Zeit, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 4, 0), 0, 0));
		this.add(startButton, new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 0, 0, 0), 0, 0));
		this.add(startzeit, new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 0, 0, 0), 43, 0));
		this.add(stopButton, new GridBagConstraints(3, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 0, 0, 0), 0, 0));
		this.add(stopzeit, new GridBagConstraints(4, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						3, 0, 0), 54, 0));
		this.add(bildId, new GridBagConstraints(6, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 0, 0, 0), 42, 0));
		this.add(BildIDLabel, new GridBagConstraints(5, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));*/
	}

	private void setModified() {
		parent.setModifiedState(true);
	}

	public void setValue(String label, String value) {
		if (values.containsKey(label))
			values.get(label).setText(value);
	}

	public String getValue(String label) {
		return values.containsKey(label) ? values.get(label).getText() : null;
	}
}
