package gui.report.image;

import io.ImageCache;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.media.jai.PlanarImage;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;

public class ImageViewer extends JPanel {

	private static final long serialVersionUID = 433491143037449032L;

	private PlanarImage src = null;

	private ImageDisplay dsp = null;

	private Panner panner;

	private Vector<String> files, thumbs;

	private JPanel pannerPanel = new JPanel(), centralPanel = new JPanel();

	private JComboBox imageChooser = new JComboBox();

	private JLabel annotationLabel = new JLabel();

	private JTextArea annotationEditor = new JTextArea();

	// private Point annotationPos;

	private Map<String, Map<Rectangle, String>> annotations;

	public ImageViewer(Vector<String> files, Vector<String> thumbs) {
		this.annotations = new Hashtable<String, Map<Rectangle, String>>();
		this.files = files;
		this.thumbs = thumbs;
		String[] items = new String[files.size()];
		for (int i = 0; i < items.length; i++) {
			items[i] = files.get(i);
			annotations.put(files.get(i), new Hashtable<Rectangle, String>());
		}
		imageChooser = new JComboBox(items);

		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (items.length != 0)
			loadImage(0);

	}

	public Rectangle getAnnotationRec() {
		return dsp.getAnnotationRec();
	}

	public String getAnnotationString() {
		return annotationEditor.getText();
	}

	public void setAnnotationString(String text) {
		annotationEditor.setText(text);
	}

	private void jbInit() throws Exception {

		this.setLayout(new BorderLayout());

		JPanel thumbsPanel = new JPanel(new GridBagLayout());

		JButton okButton = new JButton();

		centralPanel.setLayout(new BorderLayout());

		pannerPanel.setBorder(BorderFactory.createEtchedBorder());
		pannerPanel.setOpaque(true);
		pannerPanel.setLayout(new GridLayout());

		imageChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				imageChooser_actionPerformed();
			}
		});

		annotationEditor.setText("");
		annotationLabel.setVerifyInputWhenFocusTarget(true);
		annotationLabel.setText("(0,0)");

		annotationEditor.setMinimumSize(new Dimension(400, 150));

		okButton.setSelected(false);
		okButton.setText("OK");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				okButton_actionPerformed();
			}
		});

		thumbsPanel.add(pannerPanel, new GridBagConstraints(0, 0, 4, 4, 0.0,
				0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));

		thumbsPanel.add(new JLabel("Bild waehlen: "), new GridBagConstraints(4,
				0, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER,
				GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		thumbsPanel.add(imageChooser, new GridBagConstraints(6, 0, 10, 1, 0.0,
				0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));
		thumbsPanel.add(new JLabel("Annotation:"), new GridBagConstraints(4, 1,
				2, 6, 1.0, 1.0, GridBagConstraints.CENTER,
				GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

		JScrollPane tempscrollpane = new JScrollPane();
		tempscrollpane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		tempscrollpane.setAutoscrolls(true);
		tempscrollpane.setMinimumSize(new Dimension(0, 75));
		tempscrollpane.setPreferredSize(new Dimension(0, 100));
		tempscrollpane.getViewport().add(annotationEditor, null);

		thumbsPanel.add(tempscrollpane, new GridBagConstraints(6, 1, 10, 6,
				1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));

		thumbsPanel.add(new JLabel("Annotation einfuegen: "),
				new GridBagConstraints(4, 7, 2, 1, 1.0, 1.0,
						GridBagConstraints.CENTER, GridBagConstraints.BOTH,
						new Insets(5, 5, 5, 5), 0, 0));

		thumbsPanel.add(annotationLabel, new GridBagConstraints(6, 7, 2, 1,
				1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));

		thumbsPanel.add(okButton, new GridBagConstraints(9, 7, 1, 1, 1.0, 1.0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						5, 5, 5, 5), 0, 0));

		thumbsPanel.add(new JPanel(), new GridBagConstraints(10, 7, 1, 1, 1.0,
				1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));

		this.add(thumbsPanel, BorderLayout.NORTH);
		this.add(centralPanel, BorderLayout.CENTER);

	}

	void imageChooser_actionPerformed() {
		int picNr = imageChooser.getSelectedIndex();
		if (picNr >= 0 && picNr < files.size()) {
			loadImage(picNr);
		}
		setAnnotationString("");
		dsp.repaint();
		this.paintAll(this.getGraphics());

	}

	void loadImage(int picNr) {

		pannerPanel.removeAll();
		centralPanel.removeAll();
		src = ImageCache.loadImage(files.get(picNr));
		dsp = new ImageDisplay(src);

		dsp.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent arg0) {
				image_mouseClicked(arg0);
			}
		});

		for (Iterator<Rectangle> i = annotations.get(files.get(picNr)).keySet()
				.iterator(); i.hasNext();)
			dsp.addAnnotationGraphic(i.next());
		centralPanel.add(dsp);
		// build the panner controller with first picture
		PlanarImage thumb = ImageCache.loadThumb(thumbs.get(picNr));

		panner = new Panner(dsp, src, thumb);

		panner.setBorder(new CompoundBorder(new EtchedBorder(3),
				new LineBorder(Color.gray, 3)));
		panner.setMaximumSize(new Dimension(100, 100));
		pannerPanel.add(panner);

	}

	public void image_mouseClicked(MouseEvent e) {
		Point p = e.getPoint();

		// get real coordinates for clicking into image
		int x = p.x + dsp.getXOrigin();
		int y = p.y + dsp.getYOrigin();

		annotationLabel.setText(" (" + x + ", " + y + ")");
		// annotationPos = new Point(x, y);

		Point startPoint = e.getPoint();
		startPoint.x += dsp.getXOrigin();
		startPoint.y += dsp.getYOrigin();
		

		setAnnotationString("");
		
		Rectangle temp = null;
		for (Iterator<Rectangle> i = annotations.get(
				files.get(imageChooser.getSelectedIndex())).keySet().iterator(); i
				.hasNext();) {
			Rectangle r = i.next();
			if (temp == null)
				temp = r;
			
			if (r.contains(startPoint)
					
					&& (!temp.contains(startPoint)
							
					|| (temp.contains(startPoint) 
							&& temp.getHeight() * temp.getWidth() >= r.getHeight()
							* r.getWidth()
					
					)
					)
					
				/*	&& temp.getHeight() * temp.getWidth() >= r.getHeight()
							 r.getWidth()
							*/
			) {
				setAnnotationString(annotations.get(
						files.get(imageChooser.getSelectedIndex())).get(r));
				temp = r;
				
				
			}
		}

	}

	public void okButton_actionPerformed() {
		String text = getAnnotationString();
		if (text.length() > 0) {
			annotations.get(files.get(imageChooser.getSelectedIndex())).put(
					dsp.getAnnotationRec(), getAnnotationString());
			annotationEditor.setText("");
			dsp.addAnnotationGraphic(dsp.getAnnotationRec());
		}
	}

}
