
package gui.search;

import gui.suchen.listeners.KonzeptSelectionListener;

import io.XMLAccess;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import services.suchen.KonzeptSuchErgebnis;
import tools.Tools;


import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

public class KonzeptUI
extends JPanel
{
	private static final Cursor			HANDCURSOR	= new Cursor(Cursor.HAND_CURSOR);
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final Collection<JCheckBox>	checkBoxes	= new ArrayList<JCheckBox>();

	private KonzeptSelectionListener	listener;

	private int							y			= 1;

	private PanelBuilder				pb;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Constructors = ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public KonzeptUI(KonzeptSuchErgebnis suchErgebnis) {
		// Preconditions:
		assert suchErgebnis != null;

		// Implementation:
		setOpaque(true);
		setBackground(Color.WHITE);

		String string = suchErgebnis.getResultat();
		Document doc = XMLAccess.string2xml(string);
		NodeList concepts;
		try {
			concepts = XPathAPI.selectNodeList(doc, "/result/conceptName/@value");
		}
		catch (TransformerException e1) {
			throw new RuntimeException(e1);
		}

		Set<String> nodes = new HashSet<String>();
		for (int i = 0; i < concepts.getLength(); i++) {
			nodes.add(concepts.item(i).getNodeValue());
		}

		List<String> names = new ArrayList<String>(nodes);
		Collections.sort(names, new Comparator<String>() {
			public int compare(String o1, String o2) {
				return Tools.localName(o1).compareToIgnoreCase(Tools.localName(o2));
			}
		});

		FormLayout layout = new FormLayout("f:p:g,r:m:n", "");

		pb = new PanelBuilder(layout);
		for (String uri : names) {
			addKonzept(uri, false);
		}

		PanelBuilder pb2 = new PanelBuilder(new FormLayout("f:p:g", "f:p:n,f:m:g"), this);
		final JPanel panel = pb.getPanel();
		panel.setOpaque(false);
		pb2.add(panel, new CellConstraints().xy(1, 1));
		final JPanel panel2 = new JPanel();
		panel2.setOpaque(false);
		pb2.add(panel2, new CellConstraints().xy(1, 2));

	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public Collection<String> getSelectedKonzepts() {
		Collection<String> result = new ArrayList<String>();

		for (JCheckBox checkBox : checkBoxes)
			if (checkBox.isSelected())
				result.add(checkBox.getActionCommand());

		return result;
	}

	// ---------------------------------------------------------------------------------------------

	public void setListener(KonzeptSelectionListener listener) {
		// Preconditions:
		assert listener != null;

		// Implementation:
		this.listener = listener;
	}

	public void addKonzept(String uri, boolean setSelected) {
		pb.appendRow("m");
		final JCheckBox checkBox = new JCheckBox(Tools.localName(uri));
		checkBox.setActionCommand(uri);
		checkBoxes.add(checkBox);
		checkBox.setOpaque(false);

		final CellConstraints cc = new CellConstraints();

		pb.add(checkBox, cc.xy(1, y));
		JButton button = new JButton();
		URL resource = getClass().getResource("/images/arrow.png");
		button.setIcon(new ImageIcon(resource));
		button.setBorderPainted(false);
		button.setOpaque(false);
		button.setContentAreaFilled(false);
		button.setCursor(HANDCURSOR);
		button.setMinimumSize(new Dimension(20, button.getMinimumSize().height - 4));
		button.setActionCommand(uri);
		checkBox.setSelected(setSelected);
		checkBox.setForeground(Color.BLACK);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (listener != null)
					listener.onConceptChosen(e.getActionCommand());
			}
		});

		pb.add(button, cc.xy(2, y));
		y++;
	}
}