package gui.search;

import gui.suchen.graph.KonzeptModel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.Callable;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.suchen.ReportSuchKonfiguration;
import tools.Tools;
import tools.Enums.Relation;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import controll.MainControll;
import controll.RelationCacheControll;

public class ReportSuchDetailsUI extends JPanel {
	private SuchErgebnisUI parent;
	
	public ReportSuchDetailsUI(String string,
			ReportSuchKonfiguration konfiguration, SuchErgebnisUI parent) {
		FormLayout layout = new FormLayout("f:m:g", "f:m:g");
		PanelBuilder pb = new PanelBuilder(layout, this);
		this.parent = parent;
		final JTabbedPane tabbedPane = new JTabbedPane();

		for (Relation relation : Relation.values())
			tabbedPane.add(relation.getDisplayName(), new JScrollPane(
					new Details(relation, string, konfiguration)));

		pb.add(tabbedPane, new CellConstraints().xy(1, 1));
	}

	public class Details extends JPanel implements Callable<KonzeptModel>,
			Comparator<String> {

		private final String string;

		public Details(Relation relation, String string,
				ReportSuchKonfiguration konfiguration) {
			this.string = string;
			FormLayout layout = new FormLayout("3dlu,"
					+ "3dlu,f:m:g,3dlu,f:m:n," + "5dlu,"
					+ "3dlu,f:m:g,3dlu,f:m:n," + "3dlu", "f:m:n");
			layout.setColumnGroups(new int[][] { { 3, 8 } });
			PanelBuilder pb = new PanelBuilder(layout, this);
			CellConstraints cc = new CellConstraints();

			pb.addSeparator("Subklassen", cc.xyw(2, 1, 4));
			pb.addSeparator("Superklassen", cc.xyw(7, 1, 4));

			pb.appendRow("2dlu");

			
			final List<String> parents = RelationCacheControll.findParents(
					relation, string);

			final List<String> children = RelationCacheControll.findChildren(
					relation, string);

			Collections.sort(parents, this);
			Collections.sort(children, this);

			final int max = Math.max(parents.size(), children.size());

			for (int i = 1; i <= max; i++) {
				pb.appendRow("2dlu");
				pb.appendRow("m:n");
			}

			pb.appendRow("f:m:g");

			int y = 4;
			for (String child : children) {
				pb.add(new JLabel(Tools.localName(child)), cc.xy(3, y));
				pb.add(new XButton(child, konfiguration), cc.xy(5, y));
				y += 2;
			}

			y = 4;
			for (String parent : parents) {
				pb.add(new JLabel(Tools.localName(parent)), cc.xy(8, y));
				pb.add(new XButton(parent, konfiguration), cc.xy(10, y));
				y += 2;
			}
		}

		public KonzeptModel call() throws Exception {
			return new KonzeptModel(string);
		}

		public int compare(String o1, String o2) {
			return o1.compareToIgnoreCase(o2);
		}

		public class XButton extends JButton implements ActionListener {

			private final ReportSuchKonfiguration konfiguration;

			private final String konzept;

			public XButton(String konzept, ReportSuchKonfiguration konfiguration) {
				super("Befunde suchen");
				this.konfiguration = konfiguration;
				this.konzept = konzept;
				addActionListener(this);
			}

			public void actionPerformed(ActionEvent e) {
				konfiguration.setConcepts(konzept);
				
				MainControll.SERVICECOORDINATOR.requestService(
						ServiceID.PERFORM_REPORT_SEARCH, new Parameter(parent, konfiguration));
			}
		}
	}
}
