package gui.search;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.suchen.ReportSuchErgebnis;
import services.suchen.ReportSuchKonfiguration.Sortiermodus;
import tools.Tools;
import tools.Enums.Section;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import controll.MainControll;


public class ReportSuchErgebnisseUI
extends JPanel
implements
	ActionListener
{
    private static Font headFont;
    private static final Cursor HANDCURSOR = new Cursor(Cursor.HAND_CURSOR);
    
    private final ReportSuchErgebnis param;
    private final SuchErgebnisUI parent;

	public ReportSuchErgebnisseUI(ReportSuchErgebnis param, final SuchErgebnisUI parent) {
		this.param = param;
		this.parent = parent;
		
        FormLayout layout = new FormLayout("f:m:g", "f:m:g");
		PanelBuilder pb = new PanelBuilder(layout, this);

        addOverviewPane(pb);
        addTabbedPane(pb);

		setBackground(UIManager.getColor("Label.background").brighter());
		setOpaque(true);

		setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
	}
    
    protected int width()
    {
        return 5;
    }
    
    protected String head(int col)
    {
        switch (col)
        {
            case 0: return "Befund-ID";
            case 1: return "Textstelle";
            case 2: return "Alter";
            case 3: return "Geschlecht";
            case 4: return "Aehnliche Befunde";
            default:
                assert false;
                return null;
        }
    }
    
    protected JLabel body(int col, ReportInfo reportInfo)
    {
        switch (col)
        {
            case 0: return createActionLabel(reportInfo.getReportID(), ServiceID.SHOW_UI_REPORT, 
            			new Parameter(null,reportInfo.getReportID()));
            case 1: return new JLabel(reportInfo.getTextChunk());
            case 2: return new JLabel(String.valueOf(reportInfo.getAge()));
            case 3: return new JLabel(reportInfo.getGender());
            case 4: return createActionLabel("suchen", ServiceID.PERFORM_SIMILARITY_SEARCH,
            			new Parameter(parent, reportInfo.getReportID()));
            default:
                assert false;
                return null;
        }
    }
    
    protected String layoutstring()
    {
        return "3dlu,3dlu,c:m:n,5dlu,m:g,5dlu,c:m,5dlu,c:m,5dlu,c:m,3dlu";
    }
    
    protected void addOverviewPane(PanelBuilder pb)
    {
        final JPanel overview = createSummary(param);
        overview.setBackground(getBackground().brighter());
        overview.setOpaque(true);
        
        final JScrollPane scrollPane = new JScrollPane(overview);
        
        scrollPane.setBackground(UIManager.getColor("Label.background").brighter());
        scrollPane.setOpaque(true);
        scrollPane.setBorder(BorderFactory.createTitledBorder(createTitle(param)));
        
        pb.add(scrollPane, new CellConstraints().xyw(1, 1, 1));
    }
    
    protected void addTabbedPane(PanelBuilder pb)
    {
        final JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.setBorder(BorderFactory.createTitledBorder("Fundstellen in verwandten Konzepten suchen"));
        
        for (String konzept : param.getKonfiguration().getKonzepte()) {
            tabbedPane.add(Tools.localName(konzept), new ReportSuchDetailsUI(
                konzept, param.getKonfiguration(),parent));
        }
        
        pb.appendRow("3dlu");
        pb.appendRow("f:m:g(0.3)");
        pb.add(tabbedPane, new CellConstraints().xywh(
            1, 3, 1, 1, CellConstraints.FILL, CellConstraints.FILL));
    }

	private JPanel createSummary(ReportSuchErgebnis param) {
		FormLayout layout = new FormLayout(layoutstring(), "3dlu");
		PanelBuilder pb = new PanelBuilder(layout);

		final CellConstraints cc = new CellConstraints();
		int y = 1;
		pb.appendRow("m");
		pb.addLabel("Sortieren nach", cc.xyw(3, ++y, 1));
		final JComboBox comboBox = new JComboBox(Sortiermodus.values());
		comboBox.setSelectedItem(param.getKonfiguration().getSortiermodus());
		comboBox.addActionListener(this);
		pb.add(comboBox, cc.xywh(5, y, width(), 1, CellConstraints.LEFT, CellConstraints.DEFAULT));
        
        // remove Sortiermodus.NACH_AEHNLICHKEIT[...] for normal results
        if (!myServiceID().equals(ServiceID.SHOW_SIMILAR_REPORTS))
        {
            comboBox.removeItem(Sortiermodus.NACH_AEHNLICHKEIT);
            comboBox.removeItem(Sortiermodus.NACH_AEHNLICHKEIT_MIKR);
            comboBox.removeItem(Sortiermodus.NACH_AEHNLICHKEIT_MAKR);
            comboBox.removeItem(Sortiermodus.NACH_AEHNLICHKEIT_DIAG);
            comboBox.removeItem(Sortiermodus.NACH_AEHNLICHKEIT_KOMM);
        }

		for (Section section : param.getSections()) {
			pb.appendUnrelatedComponentsGapRow();
			y++;

			pb.appendRow("m");
			pb.addSeparator(createSectionTitle(section), cc.xyw(2, ++y, 2*width()));

			pb.appendRow("3dlu");
			y++;

			//final List<ReportInfo> reports = param.getReports(section);
			
			final List<ReportInfo> reports = new ArrayList<ReportInfo>(param.getReports(section));
			
			
			Collections.sort(reports, new ReportSorter(param.getKonfiguration().getSortiermodus()));
			if (reports.size() > 0) {
				pb.appendRow("m");

                // headlabels
                ++y; int x=3;
                for (int i=0; i<width(); ++i)
                {
                    JLabel headlabel = new JLabel(head(i));
                    
                    if (headFont == null)
                        headFont = headlabel.getFont().deriveFont(Font.BOLD);
                    headlabel.setFont(headFont);
                    
                    pb.add(headlabel, cc.xy(x, y));
                    
                    x += 2;
                }
               
				pb.appendRow("4dlu");
				y++;

				for (final ReportInfo reportInfo : reports) {
					pb.appendRow("m");
                    y++;
                    
                    for (int i=0; i<width(); ++i) {
                        pb.add(body(i,reportInfo), cc.xy(3+(2*i), y));
                    }
                    
					pb.appendRow("3dlu");
					y++;
				}
			}
			else {
				pb.appendRow("m");
				pb.addLabel("Keine Fundstellen.", cc.xyw(3, ++y, 7));
				pb.appendRow("3dlu");
				y++;
			}
		}

		comboBox.requestFocusInWindow();
		return pb.getPanel();
	}

	protected String createTitle(ReportSuchErgebnis param) {
		StringBuilder builder = new StringBuilder();

		final String[] konzepte = param.getKonfiguration().getKonzepte();
		int i = 1;
		final int size = konzepte.length;
		for (String konzept : konzepte) {
			builder.append(Tools.localName(konzept));
			if (i < size)
				builder.append(param.getKonfiguration().isUndVerknuepft() ? " und " : " oder ");
			i++;
		}

		final String string = "Fundstellen fuer " + builder.toString() + ":";
		return string;
	}

    protected String createSectionTitle(Section section)
    {
        return "in "+ section.getDisplayName();
    }
    
    private JLabel createActionLabel(final String text, final ServiceID service, final Parameter param)
    {
        final JLabel label = new JLabel(text);
        label.setForeground(Color.BLUE);
        label.setFont(label.getFont().deriveFont(Font.ITALIC));
        label.setCursor(HANDCURSOR);
        label.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseEntered(MouseEvent e) {

                label.setForeground(Color.ORANGE);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                label.setForeground(Color.BLUE);
            }

            @Override
            public void mouseClicked(MouseEvent e) {
                MainControll.SERVICECOORDINATOR.requestService(
                    service, param);
            }
        });
        
        return label;
    }

	public class ReportSorter
	implements
		Comparator<ReportInfo>
	{

		private final Sortiermodus	sortiermodus;

		public ReportSorter(Sortiermodus sortiermodus) {
			this.sortiermodus = sortiermodus;
		}

		@SuppressWarnings("unchecked")
		public int compare(ReportInfo o1, ReportInfo o2) {
			Comparable co1 = extract(o1);
			Comparable co2 = extract(o2);
			return co1.compareTo(co2);
		}

		private Comparable extract(ReportInfo o1) {
			switch (sortiermodus) {
				case NACH_ALTER:
					return o1.getAge();
				case NACH_GESCHLECHT:
					return o1.getGender();
				case NACH_ID:
					return o1.getReportID();
                case NACH_AEHNLICHKEIT:
                    // use negative value, to get reverse order
                    return -o1.getSimilarity();
                case NACH_AEHNLICHKEIT_MIKR:
                    return -o1.getSimilarity(Section.MIKROSKOPIE);
                case NACH_AEHNLICHKEIT_MAKR:
                    return -o1.getSimilarity(Section.MAKROSKOPIE);
                case NACH_AEHNLICHKEIT_DIAG:
                    return -o1.getSimilarity(Section.DIAGNOSE);
                case NACH_AEHNLICHKEIT_KOMM:
                    return -o1.getSimilarity(Section.KOMMENTAR);
				default:
					return null;
			}
		}
	}
    
    protected ServiceID myServiceID()
    {
        return ServiceID.SHOW_FOUND_REPORTS;
    }

	public void actionPerformed(ActionEvent e) {
		final JComboBox source = (JComboBox) e.getSource();
		final Sortiermodus modus = (Sortiermodus) source.getSelectedItem();

		param.getKonfiguration().setSortiermodus(modus);

		MainControll.SERVICECOORDINATOR.requestService(myServiceID(), new Parameter(parent,param));

	}
}