package gui.search;

import javax.swing.JLabel;

import services.coordination.ServiceID;
import services.similarity.SimilarReportsSearch.SimilarReportsSearchResult;
import services.suchen.ReportSuchErgebnis;
import tools.Enums.Section;

import com.jgoodies.forms.builder.PanelBuilder;

/**
 * Adapted result view for similarity searches.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class SimilarReportsSuchErgebnisseUI extends ReportSuchErgebnisseUI
{
    
    public SimilarReportsSuchErgebnisseUI(ReportSuchErgebnis param, SuchErgebnisUI parent)
    {
        super(param, parent);
    }

    @Override
    protected void addTabbedPane(PanelBuilder pb)
    {
        // we dont need this
    }
    
    @Override
    protected int width()
    {
        return 10;
    }
    
    @Override
    protected String head(int col)
    {
        switch (col)
        {
            case 0:
                return super.head(col);
            
            case 1:
                return "Ähnlichkeit";
                
            case 2:
            case 3:
            case 4:
            case 5:
                return Section.values()[col-2].getDisplayName().substring(0,4)+".";
                
            case 6:
                return "";
                
            case 7:
            case 8:
            case 9:
                return super.head(col-5);
                
           default:
               throw new IllegalArgumentException();
        }
         
    }
    
    @Override
    protected JLabel body(int col, ReportInfo info)
    {
        switch (col)
        {
            case 0:
                return super.body(col,info);
                
            case 1:
                return createLabel(info.getSimilarity());
            
            case 2:
            case 3:
            case 4:
            case 5:
                return createLabel(info.getSimilarity(Section.values()[col-2])); 
                
            case 6:
            case 7:
            case 8:
            case 9:
                return super.body(col-5, info);
                
            default:
                throw new IllegalArgumentException();
        }
    }
    
    @Override
    protected String layoutstring()
    {
        return "3dlu,3dlu,c:m:n,5dlu,c:m,5dlu,c:m,5dlu,c:m,5dlu,c:m,5dlu,c:m,5dlu,m:g,5dlu,c:m,5dlu,c:m,5dlu,c:m,3dlu";
    }

    @Override
    protected String createTitle(ReportSuchErgebnis param)
    {
        String original = ((SimilarReportsSearchResult) param).getOriginalReport().getReportID();
        
        return " Aehnliche Befunde zu ["+original+"] ";
    }
    
    @Override
    protected String createSectionTitle(Section _)
    {
        return "";
    }
    
    @Override
    protected ServiceID myServiceID()
    {
        return ServiceID.SHOW_SIMILAR_REPORTS;
    }
    
    private static JLabel createLabel(float f)
    {
        if (f < 0)
            return new JLabel("-");
        else
            return new JLabel(Math.round(100*f) + "%");
    }
}
