package gui.search;

import static com.jgoodies.forms.layout.CellConstraints.CENTER;
import static com.jgoodies.forms.layout.CellConstraints.LEFT;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.CompoundBorder;
import javax.swing.border.TitledBorder;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.suchen.FortschrittInfo;
import services.suchen.OntologieSuchKonfiguration;
import services.suchen.SuchService.Suchmodus;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import controll.MainControll;

public class SuchEditorUI extends JPanel {
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======= === Fields ==== ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final JTextField textField_Suchbegriffe = new JTextField(30);

	private final JButton button_Suchen = new JButton();

	private final ButtonGroup buttonGroup_BegriffsVerknuepfung = new ButtonGroup();

	private final SuchErgebnisseUI erfolgteSuchenUI = new SuchErgebnisseUI();

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======== Constructors = ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public SuchEditorUI() {
		init();
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======= === Methods === ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private void init() {
		// ----- Suche
		// -----------------------------------------------------------------------------

		textField_Suchbegriffe.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				button_Suchen.setEnabled(textField_Suchbegriffe.getText()
						.trim().length() > 0);
			}
		});

		textField_Suchbegriffe.requestFocus();

		button_Suchen.setEnabled(false);
		button_Suchen.setText("Konzepte suchen");
		button_Suchen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String sucheBegriffe = textField_Suchbegriffe.getText();
				if (sucheBegriffe == null || sucheBegriffe.trim().length() == 0)
					return;

	
				
				MainControll.SERVICECOORDINATOR.requestService(
						ServiceID.SHOW_PROGRESS,
						new Parameter(null,
						
						new FortschrittInfo(true,
								"Suche")));

				String modusAlsString = buttonGroup_BegriffsVerknuepfung
						.getSelection().getActionCommand();
				Suchmodus suchmodus = Suchmodus.valueOf(modusAlsString);
				OntologieSuchKonfiguration suchKonfiguration = new OntologieSuchKonfiguration(
						sucheBegriffe, suchmodus);

				MainControll.SERVICECOORDINATOR.requestService(
						ServiceID.RUN_ONTOLOGY_SEARCH, new Parameter (null,suchKonfiguration));

			}
		});

		// ----- Begriffsverknuepfung
		// ---------------------------------------------------------------

		JRadioButton radioButton_Wortlaut = new JRadioButton("genauem Wortlaut");
		JRadioButton radioButton_alle = new JRadioButton("allen Begriffen");
		JRadioButton radioButton_eins = new JRadioButton("mind. 1 Begriff");

		radioButton_eins.setBorder(Borders.EMPTY_BORDER);
		radioButton_alle.setBorder(Borders.EMPTY_BORDER);
		radioButton_Wortlaut.setBorder(Borders.EMPTY_BORDER);

		radioButton_Wortlaut.setActionCommand(Suchmodus.GENAU.toString());
		radioButton_alle.setActionCommand(Suchmodus.UND.toString());
		radioButton_eins.setActionCommand(Suchmodus.ODER.toString());

		buttonGroup_BegriffsVerknuepfung.add(radioButton_alle);
		buttonGroup_BegriffsVerknuepfung.add(radioButton_eins);
		buttonGroup_BegriffsVerknuepfung.add(radioButton_Wortlaut);

		radioButton_eins.setSelected(true);

		// ----- Begriffssch�rfe
		// -------------------------------------------------------------------

		DefaultBoundedRangeModel boundedRangeModel = new DefaultBoundedRangeModel(
				0, 0, 0, 10);
		boundedRangeModel.setValueIsAdjusting(true);
		Dictionary<Integer, JLabel> dic = new Hashtable<Integer, JLabel>();
		dic.put(0, new JLabel("genau"));
		dic.put(5, new JLabel("mittel"));
		dic.put(10, new JLabel("unscharf"));

		// ----- Ergebnisse
		// ------------------------------------------------------------------------

		erfolgteSuchenUI.setMinimumSize(new Dimension(400, 21));
		erfolgteSuchenUI.setPreferredSize(new Dimension(400, 21));

		// ----- Layout
		// ----------------------------------------------------------------------------

		FormLayout formLayout = new FormLayout(
				"5dlu,l:p:n,5dlu,l:p:n,3dlu,l:p:n,3dlu,l:p:n,3dlu,r:p:g,5dlu,l:p:n,5dlu,f:p:n,5dlu",
				"5dlu,f:p:n, 3dlu, f:p:n,5dlu, f:p:n, 5dlu, f:p:g, 3dlu");
		formLayout.setColumnGroups(new int[][] { { 4, 6, 8 } });

		PanelBuilder pb = new PanelBuilder(formLayout, this);
		pb.setBorder(Borders.DLU4_BORDER);

		CompoundBorder outerBorder = BorderFactory.createCompoundBorder(pb
				.getPanel().getBorder(), BorderFactory.createTitledBorder(
				BorderFactory.createCompoundBorder(BorderFactory
						.createEmptyBorder(2, 0, 0, 0), UIManager
						.getBorder("TitledBorder.border")), " Befundsuche ",
				TitledBorder.CENTER, TitledBorder.TOP));
		pb.setBorder(BorderFactory.createCompoundBorder(outerBorder,
				BorderFactory.createEmptyBorder(0, 8, 0, 8)));

		CellConstraints cc = new CellConstraints();

		pb.addLabel("Suchbegriffe:", cc.xy(2, 2));
		pb.add(textField_Suchbegriffe, cc.xyw(4, 2, 9));
		pb.add(button_Suchen, cc.xy(14, 2));

		pb.addLabel("Konzept passt zu:", cc.xy(2, 4));
		pb.add(radioButton_eins, cc.xy(4, 4, LEFT, CENTER));
		pb.add(radioButton_alle, cc.xy(6, 4, LEFT, CENTER));
		pb.add(radioButton_Wortlaut, cc.xy(8, 4, LEFT, CENTER));

		pb.addSeparator("Ergebnisse", cc.xyw(1, 6, 15));
		pb.add(erfolgteSuchenUI, cc.xyw(2, 8, 13));
	}

	// ---------------------------------------------------------------------------------------------

	@Override
	public void setVisible(boolean aFlag) {
		if (aFlag && getRootPane() != null
				&& getRootPane().getDefaultButton() != button_Suchen) {
			getRootPane().setDefaultButton(button_Suchen);
		}

		super.setVisible(aFlag);
		textField_Suchbegriffe.requestFocus();
	}

	// ---------------------------------------------------------------------------------------------

}