package gui.statistic;

import io.SQLDBAccess;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.ResultSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * <p>
 * Title: StatistikCustomPanel
 * </p>
 * <p>
 * Description: Panel, that shows customized statistics, e.g. of a certain
 * diagnosis
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author Sebastian Tietz
 * @version 1.0
 */
public class StatistikCustomPanel extends JPanel {

	/**
	 * @uml.property name="menuPanel"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	JPanel centerPanel = new JPanel();

	/**
	 * @uml.property name="group"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	ButtonGroup group = new ButtonGroup();

	/**
	 * @uml.property name="diags"
	 * @uml.associationEnd elementType="java.lang.String" multiplicity="(0 -1)"
	 */
	Vector<String> diags = new Vector<String>(),
			antigens = new Vector<String>(), findings = new Vector<String>();

	private JComboBox findingComboBox = new JComboBox(),
			markerComboBox = new JComboBox(),
			diagnosis2ComboBox = new JComboBox(),
			diagnosis1ComboBox = new JComboBox();

	public StatistikCustomPanel() {
		jbInit();
		diags = SQLDBAccess.listGroupDiseaseDiagnosis();
		for (Iterator<String> i = diags.iterator(); i.hasNext();) {
			String s = i.next();
			diagnosis1ComboBox.addItem(s.split("#")[1]);
			diagnosis2ComboBox.addItem(s.split("#")[1]);
		}

		antigens = SQLDBAccess.listGroupAntigenDiagnosis();
		for (Iterator<String> i = antigens.iterator(); i.hasNext();)
			markerComboBox.addItem(i.next().split("#")[1]);

		findings = SQLDBAccess.listGroupFindingsDiagnosis();
		for (Iterator<String> i = findings.iterator(); i.hasNext();)
			findingComboBox.addItem(i.next().split("#")[1]);

	}

	private void jbInit() {
		JCheckBox antigenOption = new JCheckBox(), findingOption = new JCheckBox(), diagnosisOption = new JCheckBox(), noOption = new JCheckBox();
		JButton weiterButton = new JButton("Weiter");
		JLabel jLabel1 = new JLabel();
		JPanel jPanel4 = new JPanel(), jPanel1 = new JPanel(
				new BorderLayout()), menuPanel = new JPanel(new BorderLayout());
		this.setLayout(new BorderLayout());
		weiterButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				weiterButton_actionPerformed();
			};
		});
		centerPanel.setLayout(new BorderLayout());

		jLabel1.setHorizontalAlignment(JLabel.CENTER);

		jLabel1.setHorizontalTextPosition(JLabel.CENTER);

		jLabel1.setText("Berichte, die folgende Diagnose enthalten:");

		jPanel1.add(jLabel1, java.awt.BorderLayout.NORTH);

		jPanel1.add(diagnosis1ComboBox, java.awt.BorderLayout.CENTER);

		JPanel temp = new JPanel(new GridLayout(5, 2));

		temp.add(new JPanel());
		temp.add(new JPanel());

		temp.add(noOption);
		temp.add(new JPanel());

		temp.add(antigenOption);
		temp.add(markerComboBox);

		temp.add(findingOption);
		temp.add(findingComboBox);

		temp.add(diagnosisOption);
		temp.add(diagnosis2ComboBox);

		jPanel4.add(weiterButton);

		menuPanel.add(jPanel1, BorderLayout.NORTH);
		menuPanel.add(temp, BorderLayout.CENTER);
		menuPanel.add(jPanel4, BorderLayout.SOUTH);

		antigenOption.setText("Bericht muss einen Marker enthalten: ");
		findingOption.setText("Bericht muss folgenden Befund enthalten: ");
		diagnosisOption
				.setText("Bericht muss folgende weitere Diagnose enthalten: ");
		noOption.setText("Keine weitere Einschraenkungen");

		group.add(diagnosisOption);
		group.add(antigenOption);
		group.add(findingOption);
		group.add(noOption);

		diagnosisOption.setSelected(false);
		noOption.setSelected(true);

		// group.setSelected(noOption, true);

		noOption.setActionCommand("no");
		diagnosisOption.setActionCommand("diag");
		antigenOption.setActionCommand("antigen");
		findingOption.setActionCommand("finding");

		this.add(menuPanel, java.awt.BorderLayout.NORTH);

		this.add(centerPanel, java.awt.BorderLayout.CENTER);

	}

	/**
	 * weiterButton_actionPerformed: main procedure, reads selected options and
	 * processes it. Finally generates chart and displays it
	 */
	public void weiterButton_actionPerformed() {
		Hashtable<Integer, Integer> age = new Hashtable<Integer, Integer>();
		int males = 0;
		int females = 0;
		int numberOfCases = 0;

		for (int i = 0; i < 10; i++) {
			age.put(new Integer(i), new Integer(0));
		}

		try {
			String q = "";
			String uri2 = "";

			// uri from diagnosis1
			String uri1 = findURI(diagnosis1ComboBox.getSelectedItem()
					.toString());

			String choice = group.getSelection().getActionCommand();
			if (choice == "no") {
				// 
				q = "select Befunde.* from Befunde, Diagnosen where Befunde.befund = Diagnosen.befund and Diagnosen.diagnose = '"
						+ uri1 + "'";
				ResultSet rs = SQLDBAccess.query(q);
				while (rs.next()) {
					int alter = rs.getInt(2);

					int klasse = alter / 10;

					age.put(klasse, age.get(klasse) + 1);

					if (rs.getString(3).equals("M"))
						males++;
					else
						females++;
				}

			} else if (choice == "diag") {
				uri2 = findURI(diagnosis2ComboBox.getSelectedItem().toString());
			} else if (choice == "antigen") {
				uri2 = findURI(markerComboBox.getSelectedItem().toString());
			} else if (choice == "finding") {
				uri2 = findURI(findingComboBox.getSelectedItem().toString());
			}

			Vector<String> matches = new Vector<String>();
			Vector<String> uris1 = SQLDBAccess.listBefundeWithDiagnosis(uri1);
			Vector<String> uris2 = SQLDBAccess.listBefundeWithDiagnosis(uri2);

			for (int i = 0; i < uris1.size(); i++) {
				String uri = uris1.get(i);
				if (uris2.contains(uri) && !matches.contains(uri))
					matches.add(uri);
			}
			for (int i = 0; i < uris2.size(); i++) {
				String uri = uris2.get(i);
				if (uris1.contains(uri) && !matches.contains(uri))
					matches.add(uri);
			}

			// get age and gender from db
			for (int i = 0; i < matches.size(); i++) {
				String bid = uris1.get(i);
				int alter = SQLDBAccess.getAgeFromBefund(bid);
				int klasse = alter / 10;
				age.put(new Integer(klasse), new Integer(age.get(
						new Integer(klasse)).intValue() + 1));
				if (SQLDBAccess.getGenderFromBefund(bid).equals("M"))
					males++;
				else
					females++;
			}

			numberOfCases = males + females;
			centerPanel.removeAll();
			this.remove(centerPanel);
			centerPanel.add(StatistikLib.makeAgeChart(age),
					java.awt.BorderLayout.CENTER);
			centerPanel.add(new JLabel("Berichte insgesamt: " + numberOfCases
					+ " - Maennlich: " + males + " - Weiblich: " + females),
					java.awt.BorderLayout.SOUTH);

			this.add(centerPanel);
			centerPanel.repaint();

			this.paintAll(this.getGraphics());
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * findURI: just concept names are displayed in the comboBoxes. this
	 * function reconstructs uri out of concept name.
	 * 
	 * @param s
	 *            String - concept name
	 * @return String - uri of concept
	 */
	private String findURI(String s) {
		for (int i = 0; i < diags.size(); i++) {
			String diag = diags.get(i);
			if (diag.endsWith("#" + s))
				return diag;
		}
		return null;
	}

}
