
package gui.suchen.graph;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import tools.Enums.Namespace;
import tools.Enums.Ontologie;
import tools.Enums.Relation;

import com.touchgraph.graphlayout.Node;

public class DefaultGraphUI
implements
	GraphUI
{

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final List<Ontologie>				angezeigteOntologien	= new ArrayList<Ontologie>();
	private final List<Relation>				angezeigteRelationen	= new ArrayList<Relation>();
	private final Map<Ontologie, Color>			ontologieZuBackground	= new HashMap<Ontologie, Color>();
	private final Map<Ontologie, Color>			ontologieZuForeground	= new HashMap<Ontologie, Color>();
	private final Map<Relation, Color>			relationZuBackground	= new HashMap<Relation, Color>();
	private boolean								antiAlialising;

	// TODO Kossi 24.09.2005: Pr�fen, ob das Locking wirklich n�tig ist.
	private final ReadWriteLock					lock					= new ReentrantReadWriteLock();

	// ---------------------------------------------------------------------------------------------

	private final Collection<GraphUIListener>	graphUIListeners		= new ArrayList<GraphUIListener>();

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Constructors = ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public DefaultGraphUI() {
		angezeigteOntologien.addAll(Arrays.asList(Ontologie.values()));
		angezeigteRelationen.addAll(Arrays.asList(Relation.values()));
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public void fromBean(PersistenteGraphUI bean) {
		// Preconditions:
		assert bean != null;

		// Implementation:
		lock.writeLock().lock();
		angezeigteOntologien.clear();
		angezeigteRelationen.clear();
		ontologieZuBackground.clear();
		ontologieZuForeground.clear();
		relationZuBackground.clear();

		for (String string : bean.getAngezeigteOntologien())
			angezeigteOntologien.add(Ontologie.fromNameSpace(Namespace.fromURI(string)));
		List<String> angezeigteRelationen2 = bean.getAngezeigteRelationen();
		for (Relation relation : Relation.values())
			if (angezeigteRelationen2.contains(relation.getUri()))
				angezeigteRelationen.add(relation);
		for (Entry<String, Color> entry : bean.getOntologieBackgrounds().entrySet())
			ontologieZuBackground.put(
				Ontologie.fromNameSpace(Namespace.fromURI(entry.getKey())), entry.getValue());
		for (Entry<String, Color> entry : bean.getOntologieForegrounds().entrySet())
			ontologieZuForeground.put(
				Ontologie.fromNameSpace(Namespace.fromURI(entry.getKey())), entry.getValue());
		for (Entry<String, Color> entry : bean.getRelationColors().entrySet())
			relationZuBackground.put(Relation.fromURI(entry.getKey()), entry.getValue());

		antiAlialising = bean.getAntialialise();
		lock.writeLock().unlock();
	}

	// ---------------------------------------------------------------------------------------------

	public boolean isVisible(Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		lock.readLock().lock();
		boolean result = angezeigteOntologien.contains(ontologie);
		lock.readLock().unlock();

		return result;
	}

	public boolean isVisible(Relation relation) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		lock.readLock().lock();
		boolean result = angezeigteRelationen.contains(relation);
		lock.readLock().unlock();

		return result;
	}

	
	public Color getOntologieForeground(Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		lock.readLock().lock();
		Color result = ontologieZuForeground.get(ontologie);
		lock.readLock().unlock();

		return result != null ? result : Color.BLACK;
	}

	public Color getOntologieBackground(Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		lock.readLock().lock();
		Color result = ontologieZuBackground.get(ontologie);
		lock.readLock().unlock();

		return result != null ? result : Color.GRAY;
	}

	public Color getRelationColor(Relation relation) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		lock.readLock().lock();
		Color result = relationZuBackground.get(relation);
		lock.readLock().unlock();

		return result != null ? result : Color.GRAY;
	}

	public boolean getAntiAlialising() {
		lock.readLock().lock();
		boolean result = antiAlialising;
		lock.readLock().unlock();

		return result;
	}

	public Collection<Relation> getAngezeigteRelationen() {
		lock.readLock().lock();
		ArrayList<Relation> result = new ArrayList<Relation>(angezeigteRelationen);
		lock.readLock().unlock();

		return result;
	}

	// ---------------------------------------------------------------------------------------------


	public void setVisibility(Ontologie ontologie, boolean visible) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:

		if (visible == isVisible(ontologie))
			return;

		lock.writeLock().lock();
		if (visible)
			angezeigteOntologien.add(ontologie);
		else
			angezeigteOntologien.remove(ontologie);

		for (GraphUIListener listener : graphUIListeners)
			listener.onVisibilityChange(ontologie, visible);
		lock.writeLock().unlock();
	}

	
	public void setVisibility(Relation relation, boolean visible) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		if (visible == isVisible(relation))
			return;

		lock.writeLock().lock();
		if (visible)
			angezeigteRelationen.add(relation);
		else
			angezeigteRelationen.remove(relation);
		lock.writeLock().unlock();

		for (GraphUIListener listener : graphUIListeners)
			listener.onVisibilityChange(relation, visible);
	}


	public void setColor(Ontologie ontologie, Color neueFarbe) {
		// Preconditions:
		assert ontologie != null;
		assert neueFarbe != null;

		// Implementation:
		lock.writeLock().lock();
		ontologieZuBackground.put(ontologie, neueFarbe);
		lock.writeLock().unlock();

		for (GraphUIListener listener : graphUIListeners)
			listener.onBackgroundChange(ontologie, neueFarbe);
	}


	public void setColor(Relation relation, Color neueFarbe) {
		// Preconditions:
		assert relation != null;
		assert neueFarbe != null;

		// Implementation:
		lock.writeLock().lock();
		relationZuBackground.put(relation, neueFarbe);
		lock.writeLock().unlock();

		for (GraphUIListener listener : graphUIListeners)
			listener.onColorChange(relation, neueFarbe);
	}


	public void setForeground(Ontologie ontologie, Color neueFarbe) {
		// Preconditions:
		assert ontologie != null;
		assert neueFarbe != null;

		// Implementation:
		lock.writeLock().lock();
		ontologieZuForeground.put(ontologie, neueFarbe);
		lock.writeLock().unlock();

		for (GraphUIListener listener : graphUIListeners)
			listener.onForegroundChange(ontologie, neueFarbe);
	}


	public void setAntiAlialising(boolean neuerWert) {
		if (neuerWert == antiAlialising)
			return;

		lock.writeLock().lock();
		antiAlialising = neuerWert;
		lock.writeLock().unlock();

		for (GraphUIListener listener : graphUIListeners)
			listener.onAntiAliaseChange(neuerWert);
	}

	public void addGraphUIListener(GraphUIListener listener) {
		// Preconditions:
		assert listener != null;

		// Implementation:
		graphUIListeners.add(listener);
	}

	// ---------------------------------------------------------------------------------------------


	public void updateUI(OntologyEdge edge) {
		// Preconditions:
		assert edge != null;

		// Implementation:
		List<Relation> relations = edge.getRelations();
		if (relations.size() > 1)
			edge.setColor(Color.DARK_GRAY);
		else
			edge.setColor(getRelationColor(relations.get(0)));
	}


	public void updateUI(OntologyNode node, boolean asParent) {
		// Preconditions:
		assert node != null;

		// Implementation:
		updateUI(node);
		node.setType(asParent ? Node.TYPE_RECTANGLE : Node.TYPE_ROUNDRECT);
		node.setFont(node.getFont().deriveFont(asParent ? Font.ITALIC : Font.PLAIN));
	}


	public void updateUI(OntologyNode node) {
		// Preconditions:
		assert node != null;

		// Implementation:
		node.setNodeBorderMouseOverColor(Color.ORANGE);
		node.setBackColor(getOntologieBackground(node.getOntologie()));
		node.setTextColor(getOntologieForeground(node.getOntologie()));
		node.setType(Node.TYPE_ROUNDRECT);
		node.setFont(node.getFont().deriveFont(Font.PLAIN));
	}

	// ---------------------------------------------------------------------------------------------

	public PersistenteGraphUI asBean() {
		final PersistenteGraphUI result = new PersistenteGraphUI();

		Map<String, Color> tmpMap = new HashMap<String, Color>();
		for (Entry<Ontologie, Color> entry : ontologieZuBackground.entrySet())
			tmpMap.put(entry.getKey().getNamespace().getURI(), entry.getValue());
		result.setOntologieBackgrounds(tmpMap);

		tmpMap = new HashMap<String, Color>();
		for (Entry<Ontologie, Color> entry : ontologieZuForeground.entrySet())
			tmpMap.put(entry.getKey().getNamespace().getURI(), entry.getValue());
		result.setOntologieForegrounds(tmpMap);

		tmpMap = new HashMap<String, Color>();
		for (Entry<Relation, Color> entry : relationZuBackground.entrySet())
			tmpMap.put(entry.getKey().getUri(), entry.getValue());
		result.setRelationColors(tmpMap);

		List<String> tmpList = new ArrayList<String>();
		for (Ontologie ontologie : angezeigteOntologien)
			tmpList.add(ontologie.getNamespace().getURI());
		result.setAngezeigteOntologien(tmpList);

		tmpList = new ArrayList<String>();
		for (Relation relation : angezeigteRelationen)
			tmpList.add(relation.getUri());
		result.setAngezeigteRelationen(tmpList);

		result.setAntialialise(antiAlialising);
		return result;
	}
}