
package gui.suchen.graph;

import gui.search.ReportInfo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Panel;
import java.awt.PopupMenu;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.suchen.ReportSuchErgebnis;
import tools.Enums.Ontologie;
import tools.Enums.Section;

import com.touchgraph.graphlayout.Edge;
import com.touchgraph.graphlayout.GLPanel;
import com.touchgraph.graphlayout.Node;
import com.touchgraph.graphlayout.TGLensSet;
import com.touchgraph.graphlayout.interaction.GLEditUI;
import com.touchgraph.graphlayout.interaction.GLNavigateUI_Hook;
import com.touchgraph.graphlayout.interaction.HVScroll;
import com.touchgraph.graphlayout.interaction.HyperScroll;
import com.touchgraph.graphlayout.interaction.LocalityScroll;
import com.touchgraph.graphlayout.interaction.PopupListener;
import com.touchgraph.graphlayout.interaction.RotateScroll;
import com.touchgraph.graphlayout.interaction.TGUIManager;
import com.touchgraph.graphlayout.interaction.ZoomScroll;

import controll.MainControll;

public class GrafischeSucheUI
extends GLPanel
implements
	GraphControlListener,
	PopupListener
{

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private GLNavigateUI_Hook			navigateUI;
	private Menu						menu_reports;
	private MenuItem					menu_add;
	protected OntologySelectListener	selectListener;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Constructors = ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public GrafischeSucheUI() {
		Edge.setEdgeDefaultLength(120);
		Edge.setEdgeMouseOverColor(Color.ORANGE);
		Edge.setEdgeDefaultColor(Color.LIGHT_GRAY);
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	@Override
	public void initialize() {
		tgPanel = new MyTGPanel(this, MainControll.DEFAULT_GRAPH_UI);
		tgLensSet = new TGLensSet();

		tgPanel.setLensSet(tgLensSet);
		hvScroll = new HVScroll(tgPanel, tgLensSet);
		zoomScroll = new ZoomScroll(tgPanel);
		hyperScroll = new HyperScroll(tgPanel);
		rotateScroll = new RotateScroll(tgPanel);
		localityScroll = new LocalityScroll(tgPanel);

		buildPanel();
		buildLens();
		addUIs();
		setVisible(true);
	}

	@Override
	public void addUIs() {
		menu_reports = new Menu("Befunde");
		menu_add = new MenuItem("Fuer Suche uebernehmen");
		menu_add.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {

				final Node mouseOverNode = tgPanel.getMouseOverN();
				if (mouseOverNode != null)
					selectListener.onOntologySelect(mouseOverNode.getURL());
			}
		});
		navigateUI = new GLNavigateUI_Hook(this, this);
		customizeNodePopup(navigateUI.getNodePopup());

		tgUIManager = new TGUIManager();
		tgUIManager.addUI(new GLEditUI(this), "Edit");

		tgUIManager.addUI(navigateUI, "Navigate");
		tgUIManager.activate("Navigate");
	}

	/*
	 * Hook, um das normale Steuerungspanel auszutauschen gegen das eigene ControlPanel.
	 */

	@Override
	protected Panel scrollSelectPanel(String[] scrollBarNames) {
		List<Ontologie> list = new ArrayList<Ontologie>(Arrays.asList(Ontologie.values()));
		list.remove(Ontologie.INDIVUDUALS);

		TouchGraphControl touchGraphControl = new TouchGraphControl(new TouchGraphKonfiguration(
			getZoomValue(), 5, MainControll.DEFAULT_GRAPH_UI));
		touchGraphControl.addGraphControlListener(this);

		final ScrollPane scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scrollPane.add(touchGraphControl);

		scrollPane.setPreferredSize(new Dimension(
			scrollPane.getPreferredSize().width, touchGraphControl.getPreferredSize().height + 25));

		Panel result = new Panel(new BorderLayout());
		result.add(scrollPane, BorderLayout.CENTER);
	
		return result;

	}

	private void customizeNodePopup(PopupMenu nodePopup) {
		// Preconditions:
		assert nodePopup != null;

		// Implementation:
		Collection<MenuItem> backup = new ArrayList<MenuItem>();
		for (int i = 0; i < nodePopup.getItemCount(); i++)
			backup.add(nodePopup.getItem(i));

		nodePopup.removeAll();

		nodePopup.add(menu_reports);
		nodePopup.add(menu_add);
		nodePopup.addSeparator();

		for (MenuItem item : backup) {
			item.setLabel(uebersetze(item.getLabel()));
			nodePopup.add(item);
		}
	}

	// ---------------------------------------------------------------------------------------------

	private String uebersetze(String label) {
		if (label.equals("Expand Node"))
			return "Ausklappen";
		if (label.equals("Collapse Node"))
			return "Einklappen";
		if (label.equals("Hide Node"))
			return "Ausblenden";
		if (label.equals("Center Node"))
			return "Zentrieren";
		return label;
	}

	private MyTGPanel myTGPanel() {
		return ((MyTGPanel) tgPanel);
	}

	// ---------------------------------------------------------------------------------------------

	public void setSelectListener(OntologySelectListener selectListener) {
		this.selectListener = selectListener;
	}

	// ---------------------------------------------------------------------------------------------

	/*
	 * Leerimplementierung, um die �berfl�ssigen Hokuspokus der Supermethode auszulassen. Dort wird
	 * zun�chst ein Zufallsgraph erzeugt, den brauchen wir nicht.
	 */
	@Override
	public void randomGraph() {}

	public void center(String concept) {
		// Preconditions:
		assert concept != null;

		// Implementation:
		myTGPanel().center(concept);
	}

	// ---------------------------------------------------------------------------------------------

	public void onRadiusChange(int value) {
		myTGPanel().setVisibleRadius(value);
	}

	public void onZoomChange(int value) {
		getZoomScroll().setZoomValue(value);
		tgPanel.repaintAfterMove();
	}

	public void onRotationChange(int value) {
		RotateScroll rotateScroll2 = getRotateScroll();
		int rotationAngle = rotateScroll2.getRotationAngle();

		int i = value - rotationAngle;
		double d = i * (2 * Math.PI / 360);
		rotateScroll2.setRotationAngle(value);
		rotateScroll2.incrementRotateAngle(d);

		// rotateScroll2.getRotateSB().setValue(value);
		tgPanel.repaintAfterMove();
	}

	public void onHyperbolicChange(int value) {
		getHyperScroll().getHyperSB().setValue(value);
		tgPanel.repaintAfterMove();
	}

	public void onPopupTriggered() {
		OntologyNode mouseOverNode = (OntologyNode) tgPanel.getMouseOverN();

		if (mouseOverNode != null) {
			menu_reports.removeAll();
			boolean hasReports = mouseOverNode.hasReports();
			menu_reports.setEnabled(hasReports);
			if (hasReports) {
				ReportSuchErgebnis reportErgebnisse = mouseOverNode.getReportErgebnisse();
				for (Section section : reportErgebnisse.getSections()) {
					Menu menuSection = new Menu(section.getDisplayName());

					List<ReportInfo> reports = new ArrayList<ReportInfo>(						reportErgebnisse.getReports(section));

					Collections.sort(reports, new Comparator<ReportInfo>() {
						public int compare(ReportInfo o1, ReportInfo o2) {
							return o1.getReportID().compareTo(o2.getReportID());
						}
					});

					if (reports.size() > 0)
						menu_reports.add(menuSection);

					for (final ReportInfo info : reports) {
						final MenuItem menuItem = new MenuItem(info.getReportID());
						menuItem.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								MainControll.SERVICECOORDINATOR.requestService(
									ServiceID.SHOW_UI_REPORT, new Parameter(null,info.getReportID()));
							}
						});
						menuSection.add(menuItem);
					}
				}
			}
		}
	}
}