package gui.suchen.graph;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.concurrent.Callable;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import services.suchen.FortschrittInfo;
import tools.ConcurrentCache;
import tools.ForEach;
import tools.TimeoutThreadPool;
import tools.Tools;
import tools.Enums.Namespace;
import tools.Enums.Ontologie;
import tools.Enums.Relation;

import com.touchgraph.graphlayout.Edge;
import com.touchgraph.graphlayout.GLPanel;
import com.touchgraph.graphlayout.Node;
import com.touchgraph.graphlayout.TGException;
import com.touchgraph.graphlayout.TGPanel;

import controll.ConceptCacheControll;
import controll.MainControll;

final class MyTGPanel extends TGPanel implements GraphUIListener {

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======= === Fields ==== ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final GLPanel parent;

	private final GraphUI graphUI;

	// ---------------------------------------------------------------------------------------------

	private final TimeoutThreadPool graphLoaderExecuter = new TimeoutThreadPool();

	private int visibleRadius = 1;

	private final ConcurrentCache<Integer, OntologyEdge> edgeCache = new ConcurrentCache<Integer, OntologyEdge>();

	private final ConcurrentCache<String, OntologyNode> nodes = new ConcurrentCache<String, OntologyNode>();

	
	private static final boolean[]   BOOLEAN_VALUES          = { true, false };
	
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======== Constructors = ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public MyTGPanel(GLPanel parent, GraphUI graphUI) {
		// Preconditions:
		assert parent != null;
		assert graphUI != null;

		// Implementation:
		this.parent = parent;
		this.graphUI = graphUI;

		OntologyNode.setAntiAlialise(graphUI.getAntiAlialising());
		OntologyEdge.setAntiAlialise(graphUI.getAntiAlialising());

		graphUI.addGraphUIListener(this);
		setBackColor(Color.WHITE);
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======= === Methods === ======= ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	@Override
	public Edge findEdge(Node from, Node to) {
		// Preconditions:
		assert from != null;
		assert to != null;

		// Implementation:
		return edgeCache.get(new EdgeID(from.getURL(), to.getURL()).hashCode());
	}

	@Override
	public Node findNode(String id) {
		// Preconditions:
		assert id != null;

		// Implementation:
		return nodes.get(id);
	}

	// ---------------------------------------------------------------------------------------------

	public void setVisibleRadius(int value) {
		if (value == visibleRadius)
			return;

		this.visibleRadius = value;
		setSelect(getSelect());
	}

	@Override
	public boolean deleteNode(Node node) {
		// Preconditions:
		assert node != null;

		// Implementation:
		if (node == getSelect())
			return false;

		final boolean superDelete = super.deleteNode(node);

		nodes.remove(node.getID());
		for (OntologyEdge edge : ((OntologyNode) node).edges())
			deleteEdge(edge);

		return superDelete;
	}

	@Override
	public void deleteEdge(Edge edge) {
		// Preconditions:
		assert edge != null;

		// Implementation:
		super.deleteEdge(edge);

		edge.from.removeEdge(edge);
		edge.to.removeEdge(edge);

		edgeCache.remove(((OntologyEdge) edge).getEdgeID().hashCode());

		if (edge.from.edgeCount() == 0)
			deleteNode(edge.from);

		if (edge.to.edgeCount() == 0)
			deleteNode(edge.to);
	}

	// ---------------------------------------------------------------------------------------------

	public void center(String uri) {
		// Preconditions:
		assert uri != null;

		// Implementation:
		setSelect(findOrCreateNode(uri));
	}

	@Override
	public void setSelect(final Node node) {
		// Preconditions:
		if (node == null)
			return;

		// Implementation:
		super.setSelect(node);
		runCenter(node);
		Thread thread = new Thread(new Runnable() {
			public void run() {

				MainControll.SERVICECOORDINATOR.requestService(
						ServiceID.SHOW_PROGRESS,
						
						new Parameter(null,
						new FortschrittInfo(true,
								"Daten nachladen")));

				adjustGraph(node);
				buildGraph(node.getURL(), 0);
				graphLoaderExecuter.waitForCompletion();

				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {

				}

				adjustGraph(node);

				// Bugfix Touchgraph

				edgeCache.forEach(new ForEach<OntologyEdge>() {
					public void each(OntologyEdge each) {
						if (!each.from.isVisible() || !each.to.isVisible())
							hideEdge(each);
					}
				});

				MainControll.SERVICECOORDINATOR.requestService(
						ServiceID.SHOW_PROGRESS,
						
						new Parameter(null,
						new FortschrittInfo(false)));
			}

		});
		thread.setDaemon(true);
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
	}

	private void runCenter(final Node node) {
		// Preconditions:
		if (node == null)
			return;

		// Implementation:
		Thread thread = new Thread(new NodeCenterer(node, parent));
		thread.setDaemon(true);
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
	}

	private void adjustGraph(final Node node) {
		// Preconditions:
		assert node != null;

		// Implementation:
		try {
			setLocale(node, visibleRadius);
		} catch (TGException e) {
		}

		runCenter(node);
	}

	private void buildGraph(final String url, final int radius) {
		// Preconditions:
		assert url != null;
		assert radius >= 0 && radius <= visibleRadius + 1;

		// Implementation:

		// Rekursionsanker
		if (radius >= visibleRadius + 1)
			return;
		if (!graphUI.isVisible(Ontologie.fromNameSpace(Namespace.fromURI(Tools
				.namespace(url)))))
			return;

		// passenden Konten holen ggf. neu Anlegen
		final OntologyNode node = findOrCreateNode(url);

		// Rekursion: Breitensuche �ber alle Relationen
		for (final Relation relation : graphUI.getAngezeigteRelationen())
			for (final Boolean findParents : BOOLEAN_VALUES) {
				graphLoaderExecuter.execute(new Runnable() {
					public void run() {
						for (final String ontClass : node.getNeighbors(
								relation, findParents)) {

							Ontologie ontologie = Ontologie
									.fromNameSpace(Namespace.fromURI(Tools
											.namespace(ontClass)));
							if (!graphUI.isVisible(ontologie))
								continue;

							OntologyNode neighbor = findOrCreateNode(ontClass);
							findOrCreateEdge(node, relation, !findParents,
									neighbor);

							graphLoaderExecuter.execute(new Runnable() {
								public void run() {
									buildGraph(ontClass, radius + 1);
								}
							});
						}
					}
				});
			}
	}

	// ---------------------------------------------------------------------------------------------

	public void onBackgroundChange(final Ontologie ontologie,
			final Color neueFarbe) {
		// Preconditions:
		assert ontologie != null;
		assert neueFarbe != null;

		// Implementation:
		nodes.forEach(new ForEach<OntologyNode>() {
			public void each(OntologyNode node) {
				if (node.getOntologie().equals(ontologie))
					node.setBackColor(neueFarbe);
			}
		});

		repaint();
	}

	public void onForegroundChange(final Ontologie ontologie,
			final Color neueFarbe) {
		// Preconditions:
		assert ontologie != null;
		assert neueFarbe != null;

		// Implementation:
		nodes.forEach(new ForEach<OntologyNode>() {
			public void each(OntologyNode node) {
				MainControll.DEFAULT_GRAPH_UI.updateUI(node);
			}
		});

		

		repaint();
	}

	public void onColorChange(final Relation relation, final Color neueFarbe) {
		// Preconditions:
		assert relation != null;
		assert neueFarbe != null;

		// Implementation:
		edgeCache.forEach(new ForEach<OntologyEdge>() {
			public void each(OntologyEdge edge) {
				MainControll.DEFAULT_GRAPH_UI.updateUI(edge);
			}
		});

		repaint();
	}

	public void onVisibilityChange(final Ontologie ontologie,
			final boolean displayable) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		if (!displayable) {
			final Collection<OntologyNode> hideNodes = new ArrayList<OntologyNode>();
			nodes.forEach(new ForEach<OntologyNode>() {
				public void each(OntologyNode each) {
					if (each.getOntologie().equals(ontologie)) {
						if (each != getSelect())
							hideNodes.add(each);
					}
				}
			});

			for (final OntologyNode node : hideNodes)
				deleteNode(node);
		} else {
			setSelect(getSelect());
		}
	}

	public void onVisibilityChange(final Relation relation, boolean displayable) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		if (!displayable) {
			final Collection<OntologyEdge> hideEdges = new ArrayList<OntologyEdge>();
			edgeCache.forEach(new ForEach<OntologyEdge>() {
				public void each(OntologyEdge each) {
					Collection<Relation> relations = each.getRelations();
					Collection<Relation> angezeigteRelationen = graphUI
							.getAngezeigteRelationen();

					if (Collections.disjoint(relations, angezeigteRelationen)) {
						hideEdges.add(each);
						each.removeRelation(relation);
					}
				}
			});

			for (final OntologyEdge node : hideEdges)
				deleteEdge(node);
		}

		Node select = getSelect();

		if (select != null)
			setSelect(select);
	}

	public void onAntiAliaseChange(boolean antialise) {
		OntologyNode.setAntiAlialise(antialise);
		OntologyEdge.setAntiAlialise(antialise);

		repaint(getLocation().x, getLocation().y, getWidth(), getHeight());
	}

	// ---------------------------------------------------------------------------------------------

	private OntologyNode findOrCreateNode(final String url) {
		// Preconditions:
		assert url != null;

		// Implementation:
		OntologyNode neighbor = nodes.getOrCreate(url,
				new Callable<OntologyNode>() {
					public OntologyNode call() {
						OntologyNode ontologyNode = new OntologyNode(
								MyTGPanel.this, ConceptCacheControll
										.getOrCreate(url));
						try {
							addNode(ontologyNode);
						} catch (TGException e) {
							throw new RuntimeException(e);
						}

						return ontologyNode;
					}
				});

		graphUI.updateUI(neighbor);
		return neighbor;
	}

	private OntologyEdge findOrCreateEdge(final OntologyNode node,
			final Relation relation, boolean isRootParent, OntologyNode neighbor) {
		final OntologyNode from = isRootParent ? node : neighbor;
		final OntologyNode to = isRootParent ? neighbor : node;
		EdgeID edgeID = new EdgeID(from.getID(), to.getID());

		OntologyEdge edge = edgeCache.getOrCreate(edgeID.hashCode(),
				new Callable<OntologyEdge>() {
					public OntologyEdge call() {

						OntologyEdge ontologyEdge = new OntologyEdge(from, to,
								relation);
						addEdge(ontologyEdge);
						return ontologyEdge;
					}
				});

		graphUI.updateUI(edge);
		return edge;
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/*
	 * | = == === ==== ===== ====== ======== Inner Classes ======== ====== =====
	 * ==== === == = |
	 */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final class NodeCenterer implements Runnable {
		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Fields ==== ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private final Node node;

		private final GLPanel scroll;

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Constructors = ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private NodeCenterer(Node node, GLPanel scroll) {
			this.node = node;
			this.scroll = scroll;
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Methods === ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public synchronized void run() {
			scroll.getHVScroll().slowScrollToCenter(node);
		}
	}

	public static class EdgeID {
		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Fields ==== ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private final String uri_2;

		private final String uri_1;

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Constructors = ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public EdgeID(String uri_1, String uri_2) {
			this.uri_1 = uri_1;
			this.uri_2 = uri_2;
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Methods === ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		@Override
		public int hashCode() {
			final String string = (uri_1 + uri_2);
			final int hashCode = string.hashCode();
			return new Integer(hashCode).hashCode();
		}
	}
}
