
package gui.suchen.graph;



import gui.search.ReportInfo;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import services.suchen.ReportSuchErgebnis;
import tools.Tools;
import tools.GUITools;
import tools.Enums.Namespace;
import tools.Enums.Ontologie;
import tools.Enums.Relation;

import com.touchgraph.graphlayout.TGPanel;

public class OntologyNode
extends com.touchgraph.graphlayout.Node
{

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Static Fields ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private static boolean	antiAlialise;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Static Methods ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public static void setAntiAlialise(boolean antiAlialise) {
		OntologyNode.antiAlialise = antiAlialise;
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final MyTGPanel		parent;
	private final KonzeptModel	konzeptModel;

	// ---------------------------------------------------------------------------------------------

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Constructors = ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public OntologyNode(MyTGPanel parent, KonzeptModel konzeptModel) {
		super(konzeptModel.getURL());

		// Preconditions:
		assert parent != null;
		assert konzeptModel != null;

		// Implementation:
		this.konzeptModel = konzeptModel;
		this.parent = parent;
		konzeptModel.setListener(new LoadListener() {
			public void onLoaded() {
				repaint();
			}
		});

		setLabel(Tools.localName(konzeptModel.getURL()));
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public boolean hasReports() {
		return konzeptModel.reportsLoaded() && konzeptModel.getReports().size() > 0;
	}

	
	public Collection<ReportInfo> getReports() {
		// Preconditions:
		assert konzeptModel.reportsLoaded();

		// Implementation:
		return konzeptModel.getReports();
	}

	
	public ReportSuchErgebnis getReportErgebnisse() {
		// Preconditions:
		assert konzeptModel.reportsLoaded();

		// Implementation:
		return konzeptModel.getReportErgebnisse();
	}

	public Ontologie getOntologie() {
		return Ontologie.fromNameSpace(getNamespace());
	}

	private Namespace getNamespace() {
		return Namespace.fromURI(Tools.namespace(getID()));
	}

	public Collection<OntologyEdge> edges() {
		Collection<OntologyEdge> result = new ArrayList<OntologyEdge>();

		for (int i = 0; i < edgeCount(); i++)
			result.add((OntologyEdge) edgeAt(i));

		return result;
	}

	public String getURL() {
		return konzeptModel.getURL();
	}

	public List<String> getNeighbors(Relation relation, boolean findParents) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		return konzeptModel.getNeighbors(findParents, relation);
	}

	// ---------------------------------------------------------------------------------------------

	public void paintNodeBody(Graphics g, TGPanel tgPanel) {
		final Graphics2D graphics2D = (Graphics2D) g;
		super.paintNodeBody(
			antiAlialise ? GUITools.antiAlialise(graphics2D) : GUITools.deAntiAlialise(graphics2D), tgPanel);

		String character = konzeptModel.reportsLoaded() ? String.valueOf(getReports().size()) : "?";

		if (!konzeptModel.reportsLoaded() || getReports().size() != 0) {
			int ix = (int) drawx;
			int iy = (int) drawy;
			int h = getHeight();
			int w = getWidth();
			int tagX = ix - (w + 7) / 2 + 2 - w % 2;
			int tagY = iy - h / 2 - 2;

			paintSmallTag(g, tagX, tagY, Color.YELLOW, Color.BLACK, character);
		}
	}

	@SuppressWarnings("hiding")
	private void paintSmallTag(
		Graphics g, int x, int y, Color background, Color foreground, String text) {
		g.setColor(background);
		int tagX = x - (((text.length() - 1) * 8) / 2);
		g.fillRect(tagX, y, text.length() * 6 + 2, 8);
		g.setColor(foreground);
		g.setFont(SMALL_TAG_FONT);
		g.drawString(text, tagX + 2, y + 7);
	}

	public void repaint() {
		parent.repaint(1000);
	}
}