
package gui.suchen.graph;

import static com.jgoodies.forms.layout.CellConstraints.*;


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;

import tools.Enums.Ontologie;
import tools.Enums.Relation;


import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.*;


public class TouchGraphControl
extends JPanel
{
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= == Constants == ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private static final String					HYPERBOLIC				= "HYPERBOLIC";
	private static final String					ROTATION				= "ROTATION";
	private static final String					ZOOM					= "ZOOM";
	private static final String					RADIUS					= "RADIUS";
	private static final String					ANTIALIASING			= "Antialiasing";

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final List<GraphControlListener>	graphControlListeners	= new ArrayList<GraphControlListener>();
	private final GraphUI						graphUI;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Constructors = ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public TouchGraphControl(TouchGraphKonfiguration konfiguration) {
		// Preconditions:
		assert konfiguration != null;

		// Implementation:
		// ----- Widgets ---------------------------------------------------------------------------

		graphUI = konfiguration.getGraphUI();
		final JCheckBox checkBox_AntiAliase = createAntiAliasingButton();
		checkBox_AntiAliase.setSelected(konfiguration.getGraphUI().getAntiAlialising());

		final JSlider slider_radius = createRadiusSlider();
		final JSlider slider_zoom = createZoomSlider(konfiguration.getZoomValue());
		final JSlider slider_rotation = createRotationSlider();
		final JSlider slider_hyperbolic = createHyperbolicSlider(konfiguration.getHyberbolic());

		final List<JCheckBox> ontologieButtons = new ArrayList<JCheckBox>();
		final List<JCheckBox> relationButtons = new ArrayList<JCheckBox>();
		final List<ColorChooserWidget> ontologieBackgroundWidgets = new ArrayList<ColorChooserWidget>();
		final List<ColorChooserWidget> ontologieForegroundWidgets = new ArrayList<ColorChooserWidget>();
		final List<ColorChooserWidget> relationColorWidgets = new ArrayList<ColorChooserWidget>();

		for (final Ontologie ontologie : Ontologie.values()) {
			if (ontologie == Ontologie.INDIVUDUALS)
				continue;

			final JCheckBox checkBox_ontologie = createOntologieButton(ontologie);
			final ColorChooserWidget backgroundChooserWidget = createOntologieBackgroundChooser(ontologie);
			final ColorChooserWidget foregroundChooserWidget = createOntologieForegroundChooser(ontologie);

			checkBox_ontologie.setSelected(graphUI.isVisible(ontologie));
			ontologieButtons.add(checkBox_ontologie);
			ontologieBackgroundWidgets.add(backgroundChooserWidget);
			ontologieForegroundWidgets.add(foregroundChooserWidget);
		}

		for (final Relation relation : Relation.values()) {
			final JCheckBox checkBox_relation = createRelationButton(relation);
			final ColorChooserWidget colorChooserWidget = createRelationColorChooserWidget(relation);

			checkBox_relation.setSelected(graphUI.isVisible(relation));
			relationButtons.add(checkBox_relation);
			relationColorWidgets.add(colorChooserWidget);
		}

		// ----- Layout ----------------------------------------------------------------------------

		FormLayout layout = new FormLayout(
			"3dlu," + "3dlu,f:m:g,3dlu,8dlu,3dlu,8dlu,3dlu," + "6dlu,"
			+ "3dlu,f:m:g,3dlu,8dlu,3dlu," + "6dlu," + "3dlu,f:m:n,3dlu,f:m:g,3dlu," + "6dlu,"
			+ "3dlu,f:m:n,3dlu,f:m:g,3dlu," + "4dlu",
			"3dlu,p:n, 0dlu,p:n, 0dlu,p:n, 0dlu,p:n, 0dlu, p:n, 0dlu,p:n");

		layout.setColumnGroups(new int[][] { { 3, 11, 19, 25 }, { 17, 23 }, { 5, 7, 13 } });
		layout.setRowGroups(new int[][] { { 4, 6, 8, 10, 12 } });

		PanelBuilder pb = new PanelBuilder(layout, this);
		CellConstraints cc = new CellConstraints();

		int y = 4;
		for (JCheckBox checkBox : ontologieButtons) {
			pb.add(checkBox, cc.xy(3, y));
			y += 2;
		}

		y = 4;
		for (ColorChooserWidget colorChooserWidget : ontologieForegroundWidgets) {
			cc = cc.xy(5, y, CellConstraints.FILL, CellConstraints.FILL);
			cc.insets = new Insets(4, 0, 4, 0);
			pb.add(colorChooserWidget, cc);
			y += 2;
		}

		y = 4;
		for (ColorChooserWidget colorChooserWidget : ontologieBackgroundWidgets) {
			cc = cc.xy(7, y, CellConstraints.FILL, CellConstraints.FILL);
			cc.insets = new Insets(4, 0, 4, 0);
			pb.add(colorChooserWidget, cc);
			y += 2;
		}

		y = 4;
		for (JCheckBox checkBox : relationButtons) {
			pb.add(checkBox, cc.xy(11, y));
			y += 2;
		}

		y = 4;
		for (final ColorChooserWidget colorChooserWidget : relationColorWidgets) {
			cc = cc.xy(13, y, CellConstraints.FILL, CellConstraints.FILL);
			cc.insets = new Insets(4, 0, 4, 0);
			pb.add(colorChooserWidget, cc);
			y += 2;
		}

		pb.addSeparator("Ontologien", cc.xyw(2, 2, 7));
		pb.addSeparator("Beziehungen", cc.xyw(10, 2, 5));
		pb.addSeparator("Steuerung", cc.xyw(16, 2, 5));
		pb.addSeparator("Anzeige", cc.xyw(22, 2, 5));

		pb.addLabel("Zoom", cc.xy(17, 4));
		pb.addLabel("Rotation", cc.xy(17, 6));
		pb.addLabel("Hyperbolik", cc.xy(17, 8));
		pb.addLabel("Radius", cc.xy(23, 4));
		pb.addLabel("Rendering", cc.xy(23, 10));

		pb.add(checkBox_AntiAliase, cc.xy(25, 10, LEFT, TOP));
		pb.add(slider_zoom, cc.xy(19, 4, FILL, CENTER));
		pb.add(slider_rotation, cc.xy(19, 6, FILL, CENTER));
		pb.add(slider_hyperbolic, cc.xy(19, 8, FILL, CENTER));
		pb.add(slider_radius, cc.xywh(25, 4, 1, 3, FILL, CENTER));
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private JSlider createHyperbolicSlider(int value) {
		final JSlider result = new JSlider(new DefaultBoundedRangeModel(value, 0, 0, 108));

		result.setBorder(Borders.EMPTY_BORDER);
		result.setName(HYPERBOLIC);
		result.setValue(value);

		result.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				for (GraphControlListener listener : graphControlListeners) {
					listener.onHyperbolicChange(result.getValue());
				}
			}
		});

		return result;
	}

	private JSlider createRotationSlider() {
		final JSlider result = new JSlider(new DefaultBoundedRangeModel(160, 0, 0, 380));

		result.setBorder(Borders.EMPTY_BORDER);
		result.setName(ROTATION);

		result.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				for (GraphControlListener listener : graphControlListeners) {
					int value2 = result.getValue();
					listener.onRotationChange(value2);
				}
			}
		});

		return result;
	}

	private JSlider createZoomSlider(int value) {
		final JSlider result = new JSlider(new DefaultBoundedRangeModel(value, 0, -100, 100));

		result.setBorder(Borders.EMPTY_BORDER);
		result.setName(ZOOM);

		result.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				for (GraphControlListener listener : graphControlListeners) {
					listener.onZoomChange(result.getValue());
				}
			}
		});

		return result;
	}

	private JSlider createRadiusSlider() {
		DefaultBoundedRangeModel boundedRangeModel = new DefaultBoundedRangeModel(1, 0, 1, 5);
		boundedRangeModel.setValueIsAdjusting(true);
		Dictionary<Integer, JLabel> dic = new Hashtable<Integer, JLabel>();
		dic.put(1, new JLabel("1"));
		dic.put(2, new JLabel("2"));
		dic.put(3, new JLabel("3"));
		dic.put(4, new JLabel("4"));
		dic.put(5, new JLabel("5"));

		final JSlider result = new JSlider(boundedRangeModel);

		result.setMajorTickSpacing(5);
		result.setMinorTickSpacing(1);
		result.setLabelTable(dic);
		result.setSnapToTicks(true);
		result.setPaintLabels(true);
		result.setPaintTicks(true);
		result.setBorder(Borders.EMPTY_BORDER);
		result.setName(RADIUS);

		result.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				for (GraphControlListener listener : graphControlListeners) {
					listener.onRadiusChange(result.getValue());
				}
			}
		});

		return result;
	}

	private JCheckBox createAntiAliasingButton() {
		final JCheckBox result = new JCheckBox(ANTIALIASING);
		result.setBorder(Borders.EMPTY_BORDER);

		result.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphUI.setAntiAlialising(result.isSelected());
			}
		});

		return result;
	}

	private ColorChooserWidget createRelationColorChooserWidget(final Relation relation) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		ColorChooserWidget result = new ColorChooserWidget(graphUI.getRelationColor(relation));

		result.setTitel("Farbe fuer Relation \"" + relation.getDisplayName() + "\" waehlen");

		result.setColorListener(new ColorListener() {
			public void onColorChange(Color neueFarbe) {
				graphUI.setColor(relation, neueFarbe);

			}
		});

		return result;
	}

	private ColorChooserWidget createOntologieBackgroundChooser(final Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		final ColorChooserWidget result = new ColorChooserWidget(
			graphUI.getOntologieBackground(ontologie));

		result.setTitel("Hintergrundfarbe fuer Ontologie \"" + ontologie.getDisplayName()
						+ "\" waehlen");

		result.setColorListener(new ColorListener() {
			public void onColorChange(Color neueFarbe) {
				graphUI.setColor(ontologie, neueFarbe);

			}
		});

		return result;
	}

	private ColorChooserWidget createOntologieForegroundChooser(final Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		final ColorChooserWidget result = new ColorChooserWidget(
			graphUI.getOntologieForeground(ontologie));

		result.setTitel("Textfarbe f�r Ontologie \"" + ontologie.getDisplayName() + "\" w�hlen");

		result.setColorListener(new ColorListener() {
			public void onColorChange(Color neueFarbe) {
				graphUI.setForeground(ontologie, neueFarbe);

			}
		});

		return result;
	}

	private JCheckBox createOntologieButton(final Ontologie ontologie) {
		// Preconditions:
		assert ontologie != null;

		// Implementation:
		final JCheckBox result = new JCheckBox(ontologie.getDisplayName());

		result.setActionCommand(ontologie.toString());
		result.setBorder(Borders.EMPTY_BORDER);

		result.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphUI.setVisibility(ontologie, result.isSelected());
			}
		});

		return result;
	}

	private JCheckBox createRelationButton(final Relation relation) {
		// Preconditions:
		assert relation != null;

		// Implementation:
		final JCheckBox result = new JCheckBox(relation.getDisplayName());

		result.setActionCommand(relation.toString());
		result.setBorder(Borders.EMPTY_BORDER);

		result.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphUI.setVisibility(relation, result.isSelected());
			}
		});

		return result;
	}

	// ---------------------------------------------------------------------------------------------

	public void addGraphControlListener(GraphControlListener listener) {
		// Preconditions:
		assert listener != null;

		// Implementation:
		graphControlListeners.add(listener);
	}
}