package io;

import javax.xml.namespace.QName;
import javax.xml.rpc.Call;
import javax.xml.rpc.ParameterMode;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;



public class OntmanagerAccess {

	private static final String INIT = "init", STRING = "string",
			QUERY = "query", QUERY2 = "query2", BUILDNETABOX = "buildNetABox",
			BUILDNETTBOX = "buildNetTBox", PARAMETERTYPE = "String_1",
			EXECUTE_SQL = "executeSQL", SEARCH_BEST = "searchBest",
			GETDIAGNOSIS = "getDiagnosis",
			DELETE_ALL_INDIVIDUALS = "deleteAllIndividuals",
			ADD_REPORT = "addReport", SAVE_MODELS = "saveModels";

	private static String BODY_NAMESPACE_VALUE = "urn:Foo",
			ENCODING_STYLE_PROPERTY = "javax.xml.rpc.encodingstyle.namespace.uri",
			NS_XSD = "http://www.w3.org/2001/XMLSchema",
			URI_ENCODING = "http://schemas.xmlsoap.org/soap/encoding/";

	private static QName QNAME_TYPE_STRING = new QName(NS_XSD, STRING);

	public static void resetDB() {
		executeSQL("drop table Befunde");
		executeSQL("drop table Diagnosen");
		executeSQL("create table Befunde (befund VARCHAR(6) PRIMARY KEY NOT NULL,age INT,gender VARCHAR(1),patient VARCHAR(6),numberReports INT);");
		executeSQL("create table Diagnosen (befund VARCHAR(6),diagnose VARCHAR(200),category VARCHAR(20),individual VARCHAR(200),textChunk VARCHAR(100))");
	}

	

	private static Call cCall;

	public static void executeSQL(String s) {
		try {
			cCall.removeAllParameters();
			cCall
					.setOperationName(new QName(BODY_NAMESPACE_VALUE,
							EXECUTE_SQL));
			cCall.addParameter(PARAMETERTYPE, QNAME_TYPE_STRING,
					ParameterMode.IN);
			cCall.setReturnType(QNAME_TYPE_STRING);
			String[] params = { s };

			cCall.invoke(params);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private static Call getCall(Service service, String qNamePort,
			String endpoint) {
		try {
			QName port = new QName(qNamePort);
			Call call = service.createCall(port);
			call.setTargetEndpointAddress(endpoint);
			call.setProperty(Call.SOAPACTION_USE_PROPERTY, new Boolean(true));
			call.setProperty(Call.SOAPACTION_URI_PROPERTY, "");
			call.setProperty(ENCODING_STYLE_PROPERTY, URI_ENCODING);
			call.setReturnType(QNAME_TYPE_STRING);
			return call;
		} catch (ServiceException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	public void init() {
		try {

			cCall.setOperationName(new QName(BODY_NAMESPACE_VALUE,
					INIT));
			cCall.removeAllParameters();
			cCall.setReturnType(null);
			String[] params = {};
			cCall.invoke(params);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	

	public static void initConnection() {
		try {
			// open connection to ontmanager
			try {
			ServiceFactory factory = ServiceFactory.newInstance();
			Service cService = factory.createService(new QName(
					Constants.ONTMANAGER_SERVICE));

			cCall = getCall(cService, Constants.ONTMANAGER_PORT, Constants.ONTMANAGER_ENDPOINT);

			
				cCall.setOperationName(new QName(BODY_NAMESPACE_VALUE, INIT));
				String[] params = {};
				cCall.invoke(params);
			} catch (Exception ex) {
				/*
				 * JOptionPane.showMessageDialog(null, ex.getMessage(),
				 * "Datenbankfehler", JOptionPane.ERROR_MESSAGE, null);
				 */
				System.out.println(ex.getMessage());
				ex.printStackTrace();
			}

			cCall.setOperationName(new QName(BODY_NAMESPACE_VALUE, INIT));
			cCall.removeAllParameters();
			cCall.setReturnType(null);
			String[] params = {};
			cCall.invoke(params);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static String getDiagnosis() {
		try {
			cCall.removeAllParameters();
			cCall
					.setOperationName(new QName(BODY_NAMESPACE_VALUE,
							GETDIAGNOSIS));
			cCall.setReturnType(QNAME_TYPE_STRING);

			String[] params = {};
			String result = (String) cCall.invoke(params);

			return result;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}

	public static String addReport(String s) {
		try {

			cCall.removeAllParameters();
			cCall.setOperationName(new QName(BODY_NAMESPACE_VALUE,
					ADD_REPORT));
			cCall.addParameter(PARAMETERTYPE, QNAME_TYPE_STRING,
					ParameterMode.IN);
			cCall.setReturnType(QNAME_TYPE_STRING);

			String[] params = { s };

			String result = (String) cCall.invoke(params);
			return result;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
	
}
