package services.similarity;

import io.SQLDBAccess;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class ReportFactory
{
    /**
     * Get report with the specefied id.
     * 
     * @param id report id
     * @return related report
     */
    public static Report get(String id)
    {
        return new Report(id);
    }
    
    /**
     * Get an iterable over all reports in database.
     * 
     * @return all reports
     */
    public static Iterable<Report> getAll()
    {
        return new ReportIterator();
    }
    
    static class ReportIterator implements Iterator<Report>, Iterable<Report>
    {
        private Report next;
        private ResultSet rs;
        
        /**
         * Create report iterator.
         * Access to database is done here. 
         */
        public ReportIterator()
        {
           
                rs = SQLDBAccess.query("SELECT befund FROM Befunde;");
           
           //     rs = null;
           
                loadNext();
            
        }
        
        /**
         * Try to load next report from result set. 
         */
        private void loadNext()
        {
            try
            {
                if (rs!=null && rs.next())
                    next = ReportFactory.get(rs.getString("befund"));
                else
                    next = null;
            }
            catch (SQLException e)
            {
                next = null;
            }
        }

        /**
         * @see java.util.Iterator#hasNext()
         */
        public boolean hasNext()
        {
            return next != null;
        }

        /**
         * Get next result from database.
         * 
         * @see java.util.Iterator#next()
         */
        public Report next()
        {
            if (!hasNext())
                throw new NoSuchElementException();
            
            Report r = next;
            
            loadNext();
            
            return r;
        }

        /**
         * This operation is not implemented.
         * A call will result in a UnsupportedOperationException.
         *
         * @throws UnsupportedOperationException always
         * @see java.util.Iterator#remove()
         */
        public void remove() throws UnsupportedOperationException
        {
            throw new UnsupportedOperationException();
        }

        /**
         * @return this
         * @see java.lang.Iterable#iterator()
         */
        public Iterator<Report> iterator()
        {
            return this;
        }
        
    }
}
