
package services.suchen;

import java.util.*;

import tools.IterableWrapper;
import tools.Triple;
import tools.Enums.Relation;


import com.hp.hpl.jena.ontology.*;

public class RelationCache
{
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Static Methods ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private static boolean matches(OntProperty onProperty, Relation relation) {
		// Preconditions:
		assert onProperty != null;
		assert relation != null;

		// Implementation:
		return relation.equals(Relation.fromURI(onProperty.getURI()));
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private Map<String, Map<Relation, Set<String>>>	cache	= new HashMap<String, Map<Relation, Set<String>>>();
	private boolean									cached;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public synchronized List<String> getNeighbors(
		OntClass ontClass, boolean findParents, Relation relation) {
		// Preconditions:
		assert ontClass != null;
		assert relation != null;

		// Implementation:
		Set<String> result = new HashSet<String>();

		// F�r is-a Beziehungen findet ein Caching bereits im Ontmodel statt.
		if (relation.equals(Relation.IS_A)) {
			@SuppressWarnings("unchecked")
			Iterator<OntClass> neighbors = findParents
				? ontClass.listSuperClasses(true)
				: ontClass.listSubClasses(true);

			for (OntClass neighborClass : new IterableWrapper<OntClass>(neighbors)) {
				if (!neighborClass.isAnon())
					result.add(neighborClass.getURI());
			}

			return new ArrayList<String>(result);
		}

		// Superklassen bzgl. anderer Relation sind direkt referenziert, also leicht zu finden.
		if (findParents) {
			@SuppressWarnings("unchecked")
			IterableWrapper<OntClass> iterable = new IterableWrapper<OntClass>(
				ontClass.listSuperClasses(true));

			for (OntClass neighborClass : iterable) {
				if (neighborClass.isRestriction()
					&& neighborClass.asRestriction().isSomeValuesFromRestriction()) {
					SomeValuesFromRestriction restriction = neighborClass.asRestriction().asSomeValuesFromRestriction();
					if (matches(restriction.getOnProperty(), relation)) {
						result.add(restriction.getSomeValuesFrom().getURI());
					}
				}
			}

			return new ArrayList<String>(result);
		}

		String uri = ontClass.getURI();

		// Subklassen bzgl. anderer Relation sind nicht direkt referenziert und werden einmal
		// geladen und gecached.

		if (!cached) {
			fillCache();
		}

		Map<Relation, Set<String>> relations = cache.get(uri);
		if (relations == null) {
			return Collections.emptyList();
		}

		Set<String> list = relations.get(relation);
		if (list == null) {
			return Collections.emptyList();
		}

		return new ArrayList<String>(list);
	}

	// ---------------------------------------------------------------------------------------------

	private synchronized void fillCache() {
		// Der Zugriff auf den Cache erfolgt nur beim Neuanlegen von Knoten,
		// deshalb wurde hier weniger Synchronisationsaufwand betrieben.

		// Falls nebenl�ufig doch schon gef�llt:
		if (cached)
			return;

		for (Triple<String, Relation, String> nachbarschaft : new SuchService().findeNachbarschaften()) {
			String parent = nachbarschaft.getC();
			Map<Relation, Set<String>> map = cache.get(parent);
			if (map == null)
				cache.put(parent, map = new HashMap<Relation, Set<String>>());

			Relation relation = nachbarschaft.getB();
			Set<String> list = map.get(relation);
			if (list == null)
				map.put(relation, list = new HashSet<String>());

			String child = nachbarschaft.getA();
			if (child != null)
				list.add(child);
		}

		cached = true;
	}
}
