
package services.suchen;

import gui.search.ReportInfo;
import io.JenaLib;
import io.SQLDBAccess;
import io.XMLAccess;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import services.coordination.Parameter;
import services.coordination.ServiceID;
import tools.Constants;
import tools.Triple;
import tools.Enums.Relation;
import tools.Enums.Section;

import com.hp.hpl.jena.ontology.Individual;

import controll.MainControll;


public class SuchService
{

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= ==== Enums ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public enum Suchmodus
	{
		ODER,
		UND,
		GENAU
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======== Static Methods ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	@SuppressWarnings("unchecked")
	private static String[] tokenize(String sucheBegriffeString) {
		if (sucheBegriffeString == null)
			return null;

		Enumeration enumeration = new StringTokenizer(sucheBegriffeString);
		ArrayList<String> list = Collections.list(enumeration);
		String[] strings = new String[list.size()];
		list.toArray(strings);
		return strings;
	}

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public void runOntologieSuche(final Parameter suchKonfiguration) {
		// Preconditions:
		assert suchKonfiguration != null;

		// Implementation:
		Thread thread = new Thread(new Runnable() {
			public void run() {
				suche((OntologieSuchKonfiguration)suchKonfiguration.value);
			}
		});
		thread.setDaemon(true);
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
	}

	@SuppressWarnings("unchecked")
	public KonzeptSuchErgebnis suche(OntologieSuchKonfiguration suchKonfiguration) {
		// Preconditions:
		assert suchKonfiguration != null;

		// Implementation:
		Suchmodus suchmodus = suchKonfiguration.getSuchmodus();
		String[] suchBegriffe = tokenize(XMLAccess.normalize(suchKonfiguration.getSucheEingaben()));

		assert suchBegriffe.length > 0;

		StringBuilder whereClause = new StringBuilder();

		if (suchmodus.equals(Suchmodus.UND)) {
			for (String suchBegriff : suchBegriffe) {
				whereClause.append(" AND (?x ~~ /");
				whereClause.append(suchBegriff);
				whereClause.append("/)");
			}
		}
		else if (suchmodus.equals(Suchmodus.GENAU)) {
			whereClause.append(" AND (?x ~~ /");

			for (String suchBegriff : suchBegriffe) {
				whereClause.append(suchBegriff);
				whereClause.append("_");
			}

			whereClause.deleteCharAt(whereClause.length() - 1);
			whereClause.append("/)");
		}
		else if (suchmodus.equals(Suchmodus.ODER)) {
			whereClause.append(" AND ( ");

			for (String suchBegriff : suchBegriffe) {
				whereClause.append("(?x ~~ /");
				whereClause.append(suchBegriff);
				whereClause.append("/)");
				whereClause.append(" || ");
			}

			whereClause.delete(whereClause.length() - 4, whereClause.length());
			whereClause.append(")");
		}
		else {
			Logger.global.severe("Unbekannter Suchmodus");
			return null;
		}

		String queryString = "SELECT ?c WHERE (?c, <http://www.w3.org/2000/01/rdf-schema#label>, ?x)"
								+ whereClause.toString();

		String result = JenaLib.query(queryString);

		KonzeptSuchErgebnis konzeptSuchErgebnis = new KonzeptSuchErgebnis(
			result, suchKonfiguration.getSucheEingaben());

		MainControll.SERVICECOORDINATOR.requestService(
			ServiceID.SHOW_FOUND_ONTOLOGIES, 
			
			new Parameter(null, 	konzeptSuchErgebnis));

		return konzeptSuchErgebnis;
	}

	// ---------------------------------------------------------------------------------------------

	public Collection<Individual> getIndividuals(String conceptName) {
		// Preconditions:
		assert conceptName != null;

		// Implementation:
		String query = "SELECT ?c WHERE (?c, <http://www.w3.org/1999/02/22-rdf-syntax-ns#type>, ?x) AND (?x eq \""
						+ conceptName + "\")";

		try {
			String queryResult = JenaLib.query(query);
			NodeList concepts = XPathAPI.selectNodeList(
				XMLAccess.string2xml(queryResult), "/result/conceptName/@value");

			Collection<Individual> individuals = new ArrayList<Individual>();

			for (int i = 0; i < concepts.getLength(); i++) {
				Node node = concepts.item(i);
				Individual individual = 
					JenaLib.getIndividualFromString(node.getNodeValue());
					
					

				assert individual != null;

				if (individual != null)
					individuals.add(individual);
			}

			return individuals;
		}
		catch (TransformerException e) {
			throw new RuntimeException(e);
		}
	}

	public Collection<tools.Triple<String, Relation, String>> findeNachbarschaften() {
		// Implementation:
		String p1 = "child";
		String p2 = "relation";
		String p3 = "parent";

		String query = "SELECT ?" + p1 + ", ?" + p2 + ", ?" + p3 + " WHERE " + "(?" + p1
						+ " rdfs:subClassOf ?isSubClassOf) " + "(?isSubClassOf owl:onProperty ?"
						+ p2 + ") " + "(?isSubClassOf owl:someValuesFrom ?" + p3 + ")";

		String queryResult = JenaLib.query3(query);
		Document doc = XMLAccess.string2xml(queryResult);
		try {
			NodeList concepts = XPathAPI.selectNodeList(doc, "/result/item");

			Collection<tools.Triple<String, Relation, String>> result = new ArrayList<Triple<String, Relation, String>>();
			for (int i = 0; i < concepts.getLength(); i++) {
				Node item = concepts.item(i);
				NamedNodeMap map = item.getAttributes();

				Relation relation = Relation.fromURI(map.getNamedItem(p2).getNodeValue());

				if (relation != null)
					result.add(new Triple<String, Relation, String>(
						map.getNamedItem(p1).getNodeValue(), relation,
						map.getNamedItem(p3).getNodeValue()));
			}
			return result;
		}
		catch (TransformerException e) {
			throw new RuntimeException(e);
		}
	}

	public void runReportSuche(Parameter konfiguration) {
		MainControll.SERVICECOORDINATOR
				.requestService(
						ServiceID.SHOW_FOUND_REPORTS,
						new Parameter(
								konfiguration.source,
								sucheReports((ReportSuchKonfiguration) konfiguration.value)));
	}

	public ReportSuchErgebnis sucheReports(ReportSuchKonfiguration suchKonfiguration) {
		// Preconditions:
		assert suchKonfiguration != null;

		// Implementation:
		

		BefundMatrix befundMatrix = new BefundMatrix();

		for (String konzept : suchKonfiguration.getKonzepte())
			for (Section section : suchKonfiguration.getSuchSections()) {
				String query = "SELECT ?c WHERE (?c, <http://www.w3.org/1999/02/22-rdf-syntax-ns#type>, <"
								+ konzept
								+ ">) (?c, bb:UMLSConceptContainedBy, ?d)"
								+ "(?d, <http://www.w3.org/1999/02/22-rdf-syntax-ns#type>, bb:"
									+ section.getLocalName()+ ")"+
								"USING bb FOR <"+Constants.OWLDATA_URI+"befundbericht.owl#>";
				String queryResult = JenaLib.query(query);

				try {

					NodeList concepts = XPathAPI.selectNodeList(
						XMLAccess.string2xml(queryResult), "/result/conceptName/@value");

					
					
					for (int i = 0; i < concepts.getLength(); i++) {
						String individualURI = concepts.item(i).getNodeValue();
						if (individualURI != null) {
							try {
								final String reportID = getPathReport(Constants.OWLDATA_URI + individualURI);
								if (reportID != null) {
									ReportInfo reportInfo = new ReportInfo(reportID);

									ResultSet res = SQLDBAccess.query(
										"select textChunk from Diagnosen where befund = '"
										+ reportID + "' and individual = '" + individualURI + "'");

									
									
									if (res != null && res.next()) {
										String chunk = XMLAccess.unesc(res.getString(1));
									
										reportInfo.setTextChunk(chunk);
								
									}

									res = SQLDBAccess.query(
										"select age, gender from Befunde where befund = '"
										+ reportID + "'");

									if (res.next()) {
										int age = res.getInt(1);
										String gender = res.getString(2);
									
										reportInfo.setAge(age);
										reportInfo.setGender(gender);
									}

									befundMatrix.index(section, konzept, reportInfo);
								}
							}
							catch (SQLException e) {
								e.printStackTrace();
							}
						}
					}
				}
				catch (DOMException e) {
					throw new RuntimeException(e);
				}
				catch (TransformerException e) {
					throw new RuntimeException(e);
				}
			}

		// ---------------------------------------------------------------------------------------------

		// TODO Kossi 25.10.2005: Und - verknüpfung

		Map<Section, Set<ReportInfo>> reportInfosPerSections = erzeugeErgebnisAusMatrix(
			suchKonfiguration.getKonzepte(), suchKonfiguration.isUndVerknuepft(), befundMatrix);

		ReportSuchErgebnis result = new ReportSuchErgebnisImpl(
			suchKonfiguration, reportInfosPerSections);

		return result;
	}

	private Map<Section, Set<ReportInfo>> erzeugeErgebnisAusMatrix(
		String[] konzepte, boolean undVerknuepft, BefundMatrix befundMatrix) {

		Map<Section, Set<ReportInfo>> result = new HashMap<Section, Set<ReportInfo>>();
		for (Section section : Section.values())
			result.put(section, new HashSet<ReportInfo>());

		for (Entry<String, Map<Section, Map<String, Set<ReportInfo>>>> entry : befundMatrix.entrySet()) {
			Map<Section, Map<String, Set<ReportInfo>>> map = entry.getValue();

			boolean b = undVerknuepft
				? alleKonzepteVorhanden(konzepte, map)
				: mind1KonzepteVorhanden(konzepte, map);

			if (!b)
				continue;

			Set<Entry<Section, Map<String, Set<ReportInfo>>>> entrySet = map.entrySet();
			for (Entry<Section, Map<String, Set<ReportInfo>>> entry2 : entrySet) {
				Section section = entry2.getKey();
				Collection<Set<ReportInfo>> values = entry2.getValue().values();
				for (Set<ReportInfo> collection : values) {

					result.get(section).addAll(collection);
				}
			}

		}

		return result;
	}

	private boolean mind1KonzepteVorhanden(
		String[] konzepte, Map<Section, Map<String, Set<ReportInfo>>> map) {

		for (String konzept : konzepte) {
			for (Section section : map.keySet()) {
				Set<ReportInfo> collection = map.get(section).get(konzept);
				if (collection != null && collection.size() > 0)
					return true;
			}
		}
		return false;
	}

	private boolean alleKonzepteVorhanden(
		String[] konzepte, Map<Section, Map<String, Set<ReportInfo>>> map) {

		outer: for (String konzept : konzepte) {
			for (Section section : map.keySet()) {
				Set<ReportInfo> collection = map.get(section).get(konzept);
				if (collection != null && collection.size() > 0)
					continue outer;
			}
			return false;
		}
		return true;
	}

	public String getPathReport(String individualURI)
	throws SQLException {
		ResultSet rs = SQLDBAccess.query("select befund from Diagnosen where individual = '"
									+ individualURI + "'");
		return (rs != null && rs.next()) ? rs.getString(1) : null;
	}

	public class ReportSuchErgebnisImpl
	implements
		ReportSuchErgebnis
	{

		private final Map<Section, Set<ReportInfo>>	reportsInSections;
		private final ReportSuchKonfiguration				konfiguration;

		public ReportSuchErgebnisImpl(
			ReportSuchKonfiguration konfiguration,
			Map<Section, Set<ReportInfo>> reportsInSections) {
			this.konfiguration = konfiguration;
			this.reportsInSections = reportsInSections;
		}

		public Collection<Section> getSections() {
			List<Section> result = new ArrayList<Section>(reportsInSections.keySet());
			Collections.sort(result);
			return result;
		}

		public ReportSuchKonfiguration getKonfiguration() {
			return this.konfiguration;
		}

		public Set<ReportInfo> getReports(Section section) {
			return reportsInSections.get(section);
		}

		public Set<ReportInfo> getReports() {
			Set<ReportInfo> result = new HashSet<ReportInfo>();
			for (Section section : getSections())
				result.addAll(getReports(section));
			return result;
		}
	}

	public class BefundMatrix
	extends HashMap<String, Map<Section, Map<String, Set<ReportInfo>>>>
	{

		public void index(Section section, String konzept, ReportInfo reportInfo) {
			ensure(reportInfo.getReportID());
			Map<Section, Map<String, Set<ReportInfo>>> hashMap = get(reportInfo.getReportID());

			ensure(hashMap, section);
			Map<String, Set<ReportInfo>> hashMap2 = hashMap.get(section);

			ensure2(hashMap2, konzept);
			Set<ReportInfo> collection = hashMap2.get(konzept);
			collection.add(reportInfo);
		}

		private void ensure2(Map<String, Set<ReportInfo>> hashMap2, String konzept) {
			if (!hashMap2.containsKey(konzept))
				hashMap2.put(konzept, new HashSet<ReportInfo>());
		}

		private void ensure(
			Map<Section, Map<String, Set<ReportInfo>>> hashMap, Section section) {
			if (!hashMap.containsKey(section))
				hashMap.put(section, new HashMap<String, Set<ReportInfo>>());
		}

		private void ensure(String reportID) {
			if (!containsKey(reportID))
				put(reportID, new HashMap<Section, Map<String, Set<ReportInfo>>>());
		}
	}
}