package similarity.concepts;

import similarity.ontology.Concept;

/**
 * This class holds the result of a similarity matching and the URIs of the compared nodes.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 *
 */
public class Result implements Comparable<Result>
{
    /**
     * Concepts
     */
    private final Concept c1, c2;
    
    /**
     * Result value
     */
    private final float value;

    /**
     * @param c1 first node
     * @param c2 second noade
     * @param value result value
     */
    public Result(Concept c1, Concept c2, float value)
    {
        this.c1 = c1;
        this.c2 = c2;
        this.value = value;
    }

    public Concept getFirstConcept()
    {
        return c1;
    }

    public Concept getSecondConcept()
    {
        return c2;
    }
    
    public String getFirstURI()
    {
        return c1.getURI();
    }

    public String getSecondURI()
    {
        return c2.getURI();
    }

    public float getValue()
    {
        return value;
    }
    
    @Override
    public String toString()
    {
        return "sim("+c1+","+c2+") = "+value;
    }

    /**
     * Compares values of both results.
     *
     * @return what comparision between values returns 
     */
    public int compareTo(Result other)
    {
        return Float.compare(value,other.value);
    }
}
