package similarity.concepts.matcher;

import similarity.ontology.Concept;

/**
 * A concept matcher calculates the similarity between two concepts.
 * 
 * <br> <p> This interface and the way of using matchers is derived from the
 * Semantic Matching Framework (SemMF) written by Radoslaw Oldakowski.
 * (see <a href="http://www.wiwiss.fu-berlin.de/suhl/radek/semmf/">SemMF Homepage</a>)</p> 
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 * 
 */
public interface Matcher
{
    /**
     * Calculate similarity between both concepts.
     * Most implementations require the concepts to be in the same taxonomy.
     * 
     * @param c1 a concept
     * @param c2 another concept
     * @return the similarity between both concepts (in range [0,1])
     */
    public float calcSim(Concept c1, Concept c2);
}
