package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

/**
 * Measur of Jiang and Conrath
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 *
 */
public class JiangConrath extends AbstractTaxonomicMatcher
{
    /**
     * Returns the reciprocal of the difference between the sum of the concepts
     * information contents and the least common subsumers information content.
     * 
     * @return 1 / ( ic(c1) + ic(c2) - ( 2 * ic(lcs(c1,c2)) ) )
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, Taxonomy t) throws ConceptNotFoundException, NoPathException
    {
        double ic_c1  = c1.informationContent();
        double ic_c2  = c2.informationContent();
        double ic_lcs = c1.leastCommonSubsumer(c2).informationContent();
        
        // if there is no frequency data, ic will be 0
        if (ic_c1 == 0 || ic_c2 == 0 || ic_lcs == 0)
        {
            if (c1.equals(c2))
                return maxValue(t);
            else
                return 0;
        }
        
        assert ic_c1 > 0;
        assert ic_c2 > 0;
        assert ic_lcs > 0;
        
        // if c1 == c2 == lcs
        if (ic_c1 +ic_c2 == 2*ic_lcs)
            return maxValue(t);
        
        double jcn_distance = ic_c1 + ic_c2 - (2*ic_lcs);
        
        assert jcn_distance > 0;
        
        return 1 / jcn_distance;
    }

    /**
     * @return 1
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return 100; //TODO better max
    }
    
    @Override
    protected double normalize(double value, double max)
    {
        // TODO norm!
        
        assert value >= 0;
        
        if (value > max)
        {
            System.err.println("sim_jcn = "+value);
            value = max;
        }
        
        return super.normalize(value,max);
    }

}
