package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

import static java.lang.Math.log;

/**
 * Measure of Leacock and Chodorow
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class LeacockChodorow extends AbstractTaxonomicMatcher
{
    /**
     * Divides the path length with doubled maximum depth and returns the negative
     * logarithm of that value. 
     * 
     * @return -log( l / 2*D )
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, Taxonomy t) throws NoPathException
    {
        return -log( c1.distance(c2) / (2d * t.getMaxDepth()) );
    }
    
    /**
     * A minimal pathlength (same concept) produces the maximum value.
     * <br> 
     * sim(c,c) = -log(1/(2*D)) = log(2*D)
     * @return log(2*t.getmaxDepth())
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return -log(1d/(2d*taxonomy.getMaxDepth()));
        // java is too unprecise (see below), so we cant optimize :-(
        // -log(1/(2d*taxonomy.getMaxDepth())) != log(2d*taxonomy.getMaxDepth())
    }

}
