package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

/**
 * Measure of Lin
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class Lin extends AbstractTaxonomicMatcher
{
    /**
     * Devides the doubled information content of the least common subsumer
     * with the sum of the conepts information contents.
     * 
     * @return 2*ic(lcs(c1,c2)) / (ic(c1) + ic(c2))
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, Taxonomy t) throws ConceptNotFoundException, NoPathException
    {
        double ic_c1  = c1.informationContent();
        double ic_c2  = c2.informationContent();
        double ic_lcs = c1.leastCommonSubsumer(c2).informationContent();
        
        // if there is no frequency data, ic will be 0
        if (ic_c1 == 0 || ic_c2 == 0 || ic_lcs == 0)
        {
            if (c1.equals(c2))
                return maxValue(t);
            else
                return 0;
        }
        
        assert ic_c1 > 0;
        assert ic_c2 > 0;
        assert ic_lcs > 0;
        
        assert 2*ic_lcs <= ic_c1 + ic_c2;

        return 2*ic_lcs / (ic_c1 + ic_c2);
    }

    /**
     * Max value is 1, since the measure is already normalized
     * 
     * @return 1
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return 1d;
    }

}
