package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

import static java.lang.Math.E;
import static java.lang.Math.pow;

/**
 * Measure of Rada
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class Rada extends AbstractTaxonomicMatcher
{
    /**
     * default value for alpha
     */
    public static final double DEFAULT_ALPHA = 0.5;
    
    /**
     * default value for beta
     */
    public static final double DEFAULT_BETA = 0.3;
    
    private final double alpha, beta;
    
    /**
     * Create Rada matcher with default values.
     */
    public Rada()
    {
        this(DEFAULT_ALPHA,DEFAULT_BETA);
    }
    
    public Rada(double alpha, double beta)
    {
        this.alpha = alpha;
        this.beta = beta;
    }

    /**
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, similarity.ontology.Taxonomy t) throws ConceptNotFoundException, NoPathException
    {
        double l = c1.distance(c2) - 1; // edge distance      
        double h = t.depthOf(c1.leastCommonSubsumer(c2));

        double t1 = pow( E, beta*h ) - pow( E,-beta*h );
        double t2 = pow( E, beta*h ) + pow( E,-beta*h );
        
        return pow ( E, ( -alpha * l * t1 / t2 ) );
    }

    /**
     * This measure is already normalized
     * 
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return 1d;
    }

}
