package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

/**
 * Measure of Resnik
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class Resnik extends AbstractTaxonomicMatcher
{
    /**
     * Calculates the information content of the least common subsumer.
     * <br><br>
     * <b>Note: This measure is not normalized. Maybe it returns a value greater <code>1</code></b>
     * <br><br>
     * @return ic( lcs(c1,c2) )
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, Taxonomy t) throws ConceptNotFoundException, NoPathException
    {
        return c1.leastCommonSubsumer(c2).informationContent();
    }

    /**
     * Upper bound is the maxmimum information content value. A concept with a
     * frequency of one has the biggest possible ic.
     * 
     * ic_max = ic(c) = -log(1/N) = -(log(1)-log(N)) = log(N),
     * where freq(c)=1 and N is total number of individuals 
     * 
     * @return ic_max = log(N)
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return Math.log(taxonomy.getRoot().getTotalIndividualCount());
    }

}
