package similarity.concepts.matcher.semantic;

import similarity.ontology.*;

/**
 * Measure of Wu and Palmer
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public class WuPalmer extends AbstractTaxonomicMatcher
{
    /**
     * Divides the doubled depth of the least common subsumer with the sum of the concepts depths.
     * 
     * @return 2 * depth(lcs(c1,c2)) / ( depth(c1) + depth(c2) )
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#calcSemanticSim(similarity.ontology.Concept, similarity.ontology.Concept, similarity.ontology.Taxonomy)
     */
    @Override
    protected double calcSemanticSim(Concept c1, Concept c2, similarity.ontology.Taxonomy t)
        throws ConceptNotFoundException, NoPathException
    {
        return t.depthOf(c1.leastCommonSubsumer(c2)) * 2d / (t.depthOf(c1) + t.depthOf(c2));
    }

    /**
     * This measure is already normalized and thus has a max value of 1. 
     * 
     * @return 1
     * @see similarity.concepts.matcher.semantic.AbstractTaxonomicMatcher#maxValue(similarity.ontology.Taxonomy)
     */
    @Override
    protected double maxValue(Taxonomy taxonomy)
    {
        return 1d;
    }

}
