package similarity.concepts.matcher.string;

import similarity.concepts.matcher.Matcher;
import similarity.ontology.Concept;



/**
 * Converts the two nodes to be compared into their string representations
 * and calls the abstract method calcStringSim() with these values.
 * 
 * All string matchers should be subclass of this class.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public abstract class AbstractStringMatcher implements Matcher
{
    /**
     * @see Matcher#calcSim(Concept, Concept)
     */
    public float calcSim(Concept c1, Concept c2)
    {
        // calculate string similarity
        return calcStringSim(c1.getLabel(),c2.getLabel());
    }

    /**
     * Subclasses must implement this.
     * 
     * @param string1 string representation of queryNode
     * @param string2 string representation of resNode
     * @return similarity between the two strings
     */
    protected abstract float calcStringSim(String string1, String string2);
    
}
