package similarity.concepts.matcher.util;

import similarity.ontology.Concept;

/**
 * This interface describes a generic connection to a similarity cache.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 *
 */
public interface Cache
{
    /**
     * Determine if there is an entry for the combination of the two given
     * concepts.
     * 
     * @param c1 a concept
     * @param c2 another concept
     * @return <code>true</code> if such an entry exists,
     *         <code>false</code> otherwise 
     */
    public boolean contains(Concept c1, Concept c2);
    
    /**
     * Insert a new entry.
     * 
     * @param c1 a concept
     * @param c2 another concept
     * @param sim similarity value
     */
    public void put(Concept c1, Concept c2, float sim);

    /**
     * Get similarity value for the given concept pair.
     * Returns -1 if such an entry does not exist.
     * 
     * @param c1 a concept 
     * @param c2 another concept
     * @return similarity value, -1 if no entry
     */
    public float get(Concept c1, Concept c2);
    
    /**
     * Reset cache.
     * <b>Caution: This will delete all entries from cache</b>
     */
    public void reset();
}
