/**
 * For calculating similarity of semistructured documents based on
 * similarities of occuring concepts. 
 */
package similarity.documents;

import java.util.Map;

import similarity.ontology.Concept;

/**
 * This interface describes a semistructured document.
 * The document is divided into sections. The enum of type <Section> gives
 * all different sections for the parts.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 *
 */
public interface Document<Section extends Enum<Section>>
{
    /**
     * Get all concepts contained in this document.
     * Can (and should) include double entries.
     * 
     * @return collection of occuring concepts
     */
    public Map<Concept,Integer> getConcepts();
    
    /**
     * Get a list of all concepts for the given section.
     * Can (and should) include double entries.
     * 
     * @param section where the concepts are
     * @return collection of occuring concepts
     */
    public Map<Concept,Integer> getConcepts(Section section);
}
