package similarity.documents;

/**
 * Describes the methods of a document matcher.
 * <E> is an enumaration which describes the different parts of the documents
 * to be matched.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 */
public interface Matcher<E extends Enum<E>>
{
    /**
     * Calculate similarity between the given documents.
     * 
     * @param doc1 a document
     * @param doc2 another document
     * @return similarity between doc1 and doc2 (a value in [0,1])
     */
    public Result<E> similarity(Document<E> doc1, Document<E> doc2);
}
