package similarity.documents;

import java.util.Map;

/**
 * Stores the result for a document matching.
 * Beside overall similarity it contains the results for each part.
 * 
 * @author Rafael Grote <grote@inf.fu-berlin.de>
 *
 */
public class Result<Part extends Enum<Part>>
{
    /**
     * Overall similarity.
     */
    private final float total;
    
    /**
     * Similarities for the sections.
     */
    private final Map<Part,Float> partial;

    /**
     * @param total overall result
     * @param partial partial results
     */
    public Result(float total, Map <Part, Float> partial)
    {
        this.total = total;
        this.partial = partial;
    }

    /**
     * Get partial result value.
     * 
     * @param part a part
     * @return similarity for given part,
     *         <code>null</code> if no value for that part
     */
    public Float getPartial(Part part)
    {
        return partial.get(part);
    }

    /**
     * Get total result value.
     * 
     * @return overall similarity
     */
    public float getTotal()
    {
        return total;
    }
    
    /**
     * Compares values of both results.
     *
     * @return what comparision between values returns 
     */
    public int compareTo(Result other)
    {
        return Float.compare(total,other.total);
    }
}
