package tools;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;


/**
 * Sammlung aller Enums
 */

public class Enums {

	/**
	 * Enums aller bekannten Ontologien.
	 */

	public static enum Ontologie {

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= == Constants == ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		SWPATH02("SWPatho2", Namespace.SWPATHO2), BEFUNDBERICHT(
				"Befundbericht", Namespace.BEFUNDBERICHT), IMMUNHISTOLOGIE(
				"Immunhistologie", Namespace.IMMUNOHISTOLOGY), INDIVUDUALS(
				"Individuals", Namespace.INDIVIDUALS), UMLSSN("UMLSSN",
				Namespace.UMLSSN), SWPATHO_UPPERLEVEL("SWPatho UpperLevel",
				Namespace.SWPATHO_UPPERLEVEL);

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Fields ======== ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private static final Map<Namespace, Ontologie> namespaceToOntology = new Hashtable<Namespace, Ontologie>();

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ====== Static Constructor ===== ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		static {
			for (Ontologie ontologie : values()) {
				namespaceToOntology.put(ontologie.getNamespace(), ontologie);
			}
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Methods ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public static Ontologie fromNameSpace(Namespace namespace) {
			return namespaceToOntology.get(namespace);
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Fields ==== ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private final String displayName;

		private final Namespace namespace;

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Constructors = ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private Ontologie(String displayName, Namespace namespace) {
			// Preconditions:
			assert displayName != null;
			assert namespace != null;

			// Implementation:
			this.displayName = displayName;
			this.namespace = namespace;
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Methods === ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public String getDisplayName() {
			return displayName;
		}

		public Enums.Namespace getNamespace() {
			return namespace;
		}
	}

	// ---------------------------------------------------------------------------------------------

	public static enum Namespace {
		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= == Constants == ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		SWPATHO2(Constants.OWLDATA_URI + Constants.SWPATHO2), BEFUNDBERICHT(
				Constants.OWLDATA_URI + Constants.BEFUNDBERICHT), IMMUNOHISTOLOGY(
				Constants.OWLDATA_URI + Constants.IMMUNOHISTOLOGY), UMLSSN(
				Constants.OWLDATA_URI + Constants.UMLSSN), SWPATHO_UPPERLEVEL(
				Constants.OWLDATA_URI + Constants.SWPATHO_UPPERLEVEL), INDIVIDUALS(
				Constants.OWLDATA_URI + Constants.INDIVIDUALS);

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Fields ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private static final Map<String, Namespace> uriToNameSpace = new java.util.Hashtable<String, Namespace>();

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ====== Static Constructor ===== ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		static {
			for (Namespace namespace : values()) {
				uriToNameSpace.put(namespace.getURI(), namespace);
			}
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Methods ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public static Namespace fromURI(String nameSpace) {
			return uriToNameSpace.get(nameSpace);
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Fields ==== ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private final String uri;

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Constructors = ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private Namespace(String uri) {
			// Preconditions:
			assert uri != null;

			// Implementation:
			this.uri = uri;
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Methods === ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public String getURI() {
			return uri;
		}
	}

	// ---------------------------------------------------------------------------------------------

	public static enum Relation {
		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= == Constants == ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		IS_A("Is a", "is_a"), PHYSICAL_PART_OF("Physical part of",
				"physical_part_of"), CONCEPTUAL_PART_OF("Conceptual part of",
				"conceptual_part_of"), LOCATION_OF("Location of", "location_of"), HAS_LOCATION(
				"Has location", "has_location");

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Fields ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private static Map<String, Relation> uriToRelation = new HashMap<String, Relation>();

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ====== Static Constructor ===== ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		static {
			for (Relation relation : values()) {
				uriToRelation.put(relation.getUri(), relation);
			}
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Static Methods ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public static Relation fromURI(String uri) {
			return uriToRelation.get(uri);
		}

		public static String getNamespace() {
			return Namespace.UMLSSN.getURI();
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Fields ==== ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private String displayName;

		private String uri;

		private String localName;

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======== Constructors = ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		private Relation(String displayName, String localName) {
			// Preconditions:
			assert displayName != null;
			assert localName != null;

			// Implementation:
			this.displayName = displayName;
			this.localName = localName;
			this.uri = Namespace.UMLSSN.getURI() + localName;
		}

		/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
		/*
		 * | = == === ==== ===== ====== ======= === Methods === ======= ======
		 * ===== ==== === == = |
		 */
		/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

		public String getDisplayName() {
			return displayName;
		}

		public String getUri() {
			return uri;
		}

		public String getLocalName() {
			return localName;
		}
	}

	// ---------------------------------------------------------------------------------------------


	public enum Section {
		MIKROSKOPIE("Microscopy_Section", "Mikroskopie"), MAKROSKOPIE(
				"Macroscopy_Section", "Makroskopie"), DIAGNOSE(
				"Diagnosis_Section", "Diagnose"), KOMMENTAR("Comments_Section",
				"Kommentar");

		private final String localName;

		private final String displayName;

		private Section(String localName, String displayName) {
			this.localName = localName;
			this.displayName = displayName;
		}

		public String getLocalName() {
			return localName;
		}

		public String getDisplayName() {
			return displayName;
		}

		/**
		 * Get enum value for localName
		 * 
		 * @param localName
		 *            local name of section
		 * @return section with specified local name
		 */
		public static Section getSection(String localName) {
			for (Section s : Section.values())
				if (s.localName.equals(localName))
					return s;

			// no suitable section
			throw new IllegalArgumentException(localName);
		}
	}
}