
package tools;

import java.util.*;
import java.util.concurrent.*;


public class TimeoutThreadPool
{
	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= == Constants == ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private static final int		NUM_THREADS					= 20;
	private static final int		TIMEOUT						= 600;

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Fields ==== ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	private final ExecutorService	originalThreadsExcecutor	= Executors.newFixedThreadPool(
																	NUM_THREADS,
																	new LowPriorityThreadFactory(
																		Executors.defaultThreadFactory()));

	// ---------------------------------------------------------------------------------------------

	private final List<Future<?>>	futures						= new ArrayList<Future<?>>();

	/* | = - = - = - = - = - = - = - = - = - = /-||=||-\ = - = - = - = - = - = - = - = - = - = | */
	/* | = == === ==== ===== ====== ======= === Methods === ======= ====== ===== ==== === == = | */
	/* | = - = - = - = - = - = - = - = - = - = \-||=||-/ = - = - = - = - = - = - = - = - = - = | */

	public void execute(Runnable runnable) {
		// Preconditions:
		assert runnable != null;

		// Implementation:
		futures.add(originalThreadsExcecutor.submit(runnable));
	}

	public void waitForCompletion() {
		while (futures.size() > 0) {
			Future<?> future = futures.get(0);
			{
				try {
					future.get(TIMEOUT, TimeUnit.SECONDS);
				}
				catch (Exception e) {
					future.cancel(true);
				}
				finally {
					futures.remove(future);
				}
			}
		}

		futures.clear();
	}
}
