package xml;

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.transform.TransformerException;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class Case {

	private Document report;

	public Case(Document report) {
		if (report == null)
			report = createEmptyDocument();
		this.report = report;
		report.getDocumentElement().normalize();
		
	}

	private Document createEmptyDocument() {

		// Dokumentenobjekt erzeugen
		// Erzeugen des Wurzelelements.
		Document doc = new DocumentImpl();

		Element root = doc.createElement("levelone");
		root.setAttribute("xmlns", "urn::hl7-org/cda");
		root.setAttribute("xmlns:sciphox", "urn::sciphox-org/sciphox");
		root.setAttribute("xmlns:xsi",
				"http://www.w3.org/2001/XMLSchema-instance");
		root.setAttribute("xsi:schemaLocation",
				"urn::hl7-org/cda sciphox-cda.xsd");
		root.setAttribute("xmlns:swpatho", "urn::swpatho-org");

		Element clinic_header = doc.createElement("clinical_document_header");

		Element body = doc.createElement("body");
		root.appendChild(clinic_header);
		root.appendChild(body);
		doc.appendChild(root);

		return doc;
	}

	public Document getDocument() {
		return report;
	}

	public String getKostentraeger() {
		try {
			NodeList nl = XPathAPI.selectNodeList(report, Constants.HEADER
					+ "/patient/local_header/*/*/@V");
			if (nl.item(0) != null)
				return nl.item(0).getNodeValue();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return null;
	}

	public String getEnumber() {
		try {
			NodeList nl = XPathAPI.selectNodeList(report, Constants.HEADER
					+ "/patient/local_header/*/*/@V");
			if (nl.item(1) != null)
				return nl.item(1).getNodeValue();

		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void setNodeValue(String label, String value) {
		// value = XMLAccess.escapeHTMLcontent(value);
		if (label == null)
			return;
		try {
			String query = Constants.getQueryFromLabel(label);
			if (query == null)
				return;
			Node temp = XPathAPI.selectSingleNode(report, query);
			if (temp == null) {
				String tempString = "";
				for (StringTokenizer st = new StringTokenizer(query, "/"); st
						.hasMoreTokens();) {
					String tempString2 = st.nextToken();
					if (XPathAPI.selectSingleNode(report, tempString + "/"
							+ tempString2) == null)
						if (!tempString2.startsWith("@"))
							XPathAPI.selectSingleNode(report, tempString)
									.appendChild(
											report.createElement(tempString2));
						else
							((Element) XPathAPI.selectSingleNode(report,
									tempString)).setAttribute(tempString2
									.substring(1), null);
					tempString += "/" + tempString2;
				}
			}
			XPathAPI.selectSingleNode(report, query).setNodeValue(value);
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}

	public String getNodeValue(String label) {
		if (label == null)
			return null;
		try {
			String query = Constants.getQueryFromLabel(label);
			if (query == null)
				return null;
			Node temp = XPathAPI.selectSingleNode(report, query);
			if (temp != null)
				return 	temp.getNodeValue();
			else {
				return null;
			}
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return null;
	}

	public List<String> getCaptions() {
		List<String> result = new LinkedList<String>();
		try {
			String tempString = Constants.BODY;
			if (isSchnellSchnitt())
				tempString += "/" + Constants.SECTION;
			Node parent = XPathAPI.selectSingleNode(report, tempString);
			while (parent != null) {
				NodeList nl = XPathAPI
						.selectNodeList(parent, Constants.SECTION);
				parent = null;
				for (int i = 0; i < nl.getLength(); i++) {
					Node caption = XPathAPI.selectSingleNode(nl.item(i),
							Constants.CAPTION);
					boolean isSection = false;
					for (int j = 0; j < Constants.sections.length && !isSection; j++)
						if (caption != null
								&& Constants.sections[j].equals(caption
										.getTextContent()))
							isSection = true;
					if (!isSection) {
						result.add(caption.getTextContent());
						parent = nl.item(i);
					}
				}
			}
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return result;
	}

	public boolean isSchnellSchnitt() {
		try {
			String tempString = Constants.BODY + "/" + Constants.SECTION + "/"
					+ Constants.SECTION;
			NodeList nl = XPathAPI.selectNodeList(report, tempString);
			for (int i = 0; i < nl.getLength(); i++)
				if (XPathAPI.selectSingleNode(nl.item(i), Constants.CAPTION)
						.getTextContent().equals("Schnellschnitt"))
					return true;
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return false;
	}

	public void setSection(String section, int position) {
		try {
			String path = Constants.BODY;
			if (section.equals("Schnellschnitt") || isSchnellSchnitt()) {
				Node n = XPathAPI.selectSingleNode(report, path + "/"
						+ Constants.SECTION);
				if (n == null) {
					XPathAPI.selectSingleNode(report, path).appendChild(
							report.createElement(Constants.SECTION));
					XPathAPI.selectSingleNode(report,
							path + "/" + Constants.SECTION).appendChild(
							report.createElement(Constants.CAPTION));
					XPathAPI.selectSingleNode(
							report,
							path + "/" + Constants.SECTION + "/"
									+ Constants.CAPTION).setTextContent(
							"Befund");
				}
				path += "/" + Constants.SECTION;
			}
			Node parent = XPathAPI.selectSingleNode(report, path);
			for (; position >= 0; position--) {
				NodeList nl = XPathAPI
						.selectNodeList(parent, Constants.SECTION);
				if (position != 0) {
					for (int i = 0; i < nl.getLength(); i++) {
						Node caption = XPathAPI.selectSingleNode(nl.item(i),
								Constants.CAPTION);
						boolean foundnew = false;
						for (int j = 0; j < Constants.sections.length
								&& !foundnew; j++) {
							if (caption != null
									&& Constants.sections[j].equals(caption
											.getTextContent()))
								foundnew = true;
						}
						if (!foundnew) {
							parent = nl.item(i);
							break;
						}
					}
				}
				if (position == 0) {
					for (int i = 0; i < nl.getLength(); i++) {
						Node caption = XPathAPI.selectSingleNode(nl.item(i),
								Constants.CAPTION);
						if (caption != null
								&& section.equals(caption.getTextContent()))
							return;
					}
					Node sect = report.createElement(Constants.SECTION);
					Node cap = report.createElement(Constants.CAPTION);
					cap.setTextContent(section);
					sect.appendChild(cap);
					parent.appendChild(sect);
					return;
				}
			}
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}

	public void setContent(String section, int position, String value) {
		if (isSchnellSchnitt())
			position++;
		try {
			Node parent = XPathAPI.selectSingleNode(report, Constants.BODY
					+ "/" + Constants.SECTION);
			for (; position > 0; position--) {
				NodeList nl = XPathAPI
						.selectNodeList(parent, Constants.SECTION);
				for (int i = 0; i < nl.getLength(); i++) {
					Node caption = XPathAPI.selectSingleNode(nl.item(i),
							Constants.CAPTION);
					boolean found = false;
					for (int j = 0; j < Constants.sections.length && !found; j++) {
						if (caption != null
								&& Constants.sections[j].equals(caption
										.getTextContent())) {
							found = true;
						}
					}
					if (!found) {
						parent = nl.item(i);
						break;
					}
				}
			}
			NodeList captions = XPathAPI.selectNodeList(parent,
					Constants.SECTION);
			for (int i = 0; i < captions.getLength(); i++) {
				Node caption = XPathAPI.selectSingleNode(captions.item(i),
						Constants.CAPTION);

				for (int j = 0; j < Constants.sections.length; j++) {
					if (caption != null
							&& Constants.sections[j].equals(caption
									.getTextContent())) {
						if (section.equals(Constants.sections[j])) {
							XPathAPI.selectSingleNode(captions.item(i),
									"paragraph/content").setTextContent(value);
							return;
						}
					}
				}
			}
			Node sect = report.createElement(Constants.SECTION);
			Node cap = report.createElement(Constants.CAPTION);
			Node para = report.createElement("paragraph");
			Node content = report.createElement("content");
			cap.setTextContent(section);
			sect.appendChild(cap);
			content.setTextContent(value);
			para.appendChild(content);
			sect.appendChild(para);
			parent.appendChild(sect);
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}

	public String getContent(String section, int position) {
		int count = 0;
		try {
			String tempString = Constants.BODY + "/" + Constants.SECTION + "/"
					+ Constants.SECTION;
			for (NodeList nl = XPathAPI.selectNodeList(report, tempString); nl
					.getLength() > 0; tempString += "/" + Constants.SECTION, nl = XPathAPI
					.selectNodeList(report, tempString), count++)
				for (int i = 0; i < nl.getLength(); i++) {
					String caption = XPathAPI.selectSingleNode(nl.item(i),
							Constants.CAPTION).getTextContent();
					if (caption.equals("Schnellschnitt"))
						count--;
					else if (caption.equals(section) && position == count)
						return XPathAPI.selectSingleNode(nl.item(i),
								"paragraph/content").getTextContent();

				}
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return null;
	}

	public List<String> getSections(int position) {
		LinkedList<String> temp = new LinkedList<String>();
		int count = 0;
		try {
			String tempString = Constants.BODY + "/" + Constants.SECTION
					+ ((isSchnellSchnitt()) ? "/" + Constants.SECTION : "");
			for (NodeList nl = XPathAPI.selectNodeList(report, tempString); nl
					.getLength() > 0; tempString += "/" + Constants.SECTION, nl = XPathAPI
					.selectNodeList(report, tempString), count++)
				for (int i = 0; i < nl.getLength(); i++) {
					String caption = XPathAPI.selectSingleNode(nl.item(i),
							Constants.CAPTION).getTextContent();
					if (caption.equals("Schnellschnitt"))
						count--;
					else {
						boolean isSection = false;
						for (int j = 0; j < Constants.sections.length; j++)
							isSection |= Constants.sections[j].equals(caption);
						if (isSection && position == count - 1)
							temp.add(caption);
					}
				}
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		return temp;
	}
}